#include "animation.h"

#include <SDL/SDL.h>

#include "resourcehandler.h"

// static
vector<AnimationObject*> AnimationSystem::animation_objects;

void Outline::reset ()
{
	p1.x = -1.0;
	p1.y = -1.0;

	p2.x = -1.0;
	p2.y =  1.0;

	p3.x =  1.0;
	p3.y =  1.0;

	p4.x =  1.0;
	p4.y = -1.0;
}

Animation::~Animation ()
{
	/* must be defined */
}

void Animation::tick (float delta_t)
{
	elapsed += delta_t;
}

bool Animation::isActive ()
{


	if (elapsed < delay)
		return false;
	if (duration < 0.0)
		return true;
	if (elapsed < (delay + duration))
		return true;
	else
		return false;
}

bool Animation::isFinished ()
{
	if (duration < 0.0)	// negative duration means eternal life
		return false;

	if (elapsed > (delay + duration))
		return true;
	else
		return false;
}

AnimationObject::AnimationObject ()
	:
	idle_t (0.0)
{
	/* empty */
}

AnimationObject::~AnimationObject ()
{
	clearAnimations ();
}


void AnimationObject::step (float delta_t)
{
	idle_t += delta_t;
}

void AnimationObject::animate ()
{
	outline.reset ();
	position.x = 0.0;
	position.y = 0.0;

	vector<Animation*>::iterator it;
	for (it = applied_animations.begin (); it != applied_animations.end ();) {
		(*it)->tick (idle_t);

		if ((*it)->isActive ()) {
			(*it)->exec (outline, position, idle_t);
			it++;
		} else if ((*it)->isFinished ()) {
			//delete *it;
			it = applied_animations.erase (it);
		} else
			it++;
	}

	draw (position, outline);
	idle_t = 0.0;
}

void AnimationObject::clearAnimations  ()
{
	vector<Animation*>::iterator it = applied_animations.begin ();

	while (it != applied_animations.end ())	{
		//delete *it;
		it = applied_animations.erase (it);
	}

	applied_animations.clear ();
}

void AnimationObject::addAnimation (Animation* animation)
{
	applied_animations.push_back (animation);
}

SimpleAnimation::SimpleAnimation (string texture_path)
{
	ResourceHandler* resource_handler = ResourceHandler::instance ();
	texture = resource_handler->recvTexture (texture_path.c_str());
}

SimpleAnimation::~SimpleAnimation ()
{
	clearAnimations ();
}

void SimpleAnimation::draw (Position& pos, Outline& out)
{
		glEnable (GL_TEXTURE_2D);
		glColor4f (1.0, 1.0, 1.0, 1.0);
		glBindTexture (GL_TEXTURE_2D, texture);

		glBegin (GL_QUADS);
			glTexCoord2f (0.0, 0.0); glVertex3f (out.p2.x, out.p2.y, 0.0);
			glTexCoord2f (1.0, 0.0); glVertex3f (out.p3.x, out.p3.y, 0.0);
			glTexCoord2f (1.0, 1.0); glVertex3f (out.p4.x, out.p4.y, 0.0);
			glTexCoord2f (0.0, 1.0); glVertex3f (out.p1.x, out.p1.y, 0.0);
		glEnd ();

		glDisable (GL_TEXTURE_2D);
}

AnimationSystem::AnimationSystem ()
{
	/* empty */
}

AnimationSystem::~AnimationSystem ()
{
	/*
	vector<long, AnimationObject*>::iterator it;
	for (it = animation_objects.begin (); it != animation_objects.end (); it++) {
		delete it->second;
	}
	*/

	animation_objects.clear ();
}

void AnimationSystem::render (long id)
{
	/*
	map<long, AnimationObject*>::iterator it = animation_objects.find (id);
	if (it != animation_objects.end ()) {
		it->second->animate ();
	}
	*/
}

void AnimationSystem::update (float delta_t)
{
	vector<AnimationObject*>::iterator it;
	for (it = animation_objects.begin (); it != animation_objects.end (); it++) {
		(*it)->step (delta_t);
	}
}

void AnimationSystem::newAnimationObject (AnimationObject* animation_object)
{
	/*
	map<long, AnimationObject*>::iterator it = animation_objects.find (id);
	if (it != animation_objects.end ()) {
		animation_objects[id] = animation_object;
	}
	else {
		delete it->second;
		animation_objects[id] = animation_object;
	}
	*/

	if (animation_object != 0) {
		animation_objects.push_back (animation_object);
	}
}

/*
void AnimationSystem::applyAnimation (long id, Animation* animation)
{
	map<long, AnimationObject*>::iterator it = animation_objects.find (id);
	if (it != animation_objects.end ()) {
		it->second->addAnimation (animation);
	}
}
*/
