#ifndef ANIMATION_H
#define ANIMATION_H

#include <GL/gl.h>
#include <vector>
#include <string>
#include <map>

using namespace std;

struct Position {
	float x, y;
};

struct Outline {
	void reset ();

	Position p1, p2, p3, p4;
};

class Animation 
{
	public:

	virtual ~Animation ();

	void tick (float delta_t);
	bool isActive ();
	bool isFinished ();
	virtual void exec (Outline& outline, Position& position, float delta_t) =0;

	protected:
	
	float elapsed;
	float delay;
	float duration;
};

class AnimationObject 
{
	public:

	AnimationObject ();
	virtual ~AnimationObject ();

	void step (float delta_t);
	void animate ();
	void clearAnimations  ();
	void addAnimation (Animation* animation);
	virtual void draw (Position& position, Outline& outline) = 0;

	protected:
	float getIdleTime () { return idle_t; }

	private:
	float idle_t;

	Outline outline;
	Position position;

	vector<Animation*> applied_animations;
};

class SimpleAnimation : public AnimationObject
{
	public:

	SimpleAnimation (string texture_path);
	virtual ~SimpleAnimation ();

	void draw (Position& position, Outline& outline);

	private:
	GLuint texture;
};

class AnimationSystem 
{
	public:

	AnimationSystem ();
	~AnimationSystem ();

	static void render (long id);	// obsolete
	void update (float delta_t);

	static void newAnimationObject (AnimationObject* animation_object);
//	static void applyAnimation (long id, Animation* animation);

	private:

	static vector<AnimationObject*> animation_objects;
};
#endif // ANIMATION_H
