#include <GL/gl.h>
#include <iostream>

#include "basicanimations.h"

Rotate::Rotate ()
{
	this->direction = 0.0;
	this->duration = -1.0;
	this->delay = 0.0;

	acc = 0.0;
}

Rotate::Rotate (float direction, float duration, float delay)
{
	this->direction = direction;
	this->duration = duration;
	this->delay = delay;

	acc = 0.0;
}

void Rotate::set (float direction)
{
	this->direction = direction;
}

void Rotate::reset ()
{
	acc = 0.0;
}

void Rotate::exec (Outline& outline, Position& position, float delta_t)
{
	acc += direction * delta_t;

	glRotatef (acc, 0.0, 0.0, 1.0);
}

Scale::Scale (float sx, float sy, float duration, float delay)
{
	this->sx = sx;
	this->sy = sy;
	this->duration = duration;
	this->delay = delay;
}
	
void Scale::exec (Outline& outline, Position& position, float delta_t)
{
	float _sx = sx * ((elapsed - delay)/ duration);
	float _sy = sy * ((elapsed - delay)/ duration);

	outline.p1.x *= _sx;
	outline.p2.x *= _sx;
	outline.p3.x *= _sx;
	outline.p4.x *= _sx;

	outline.p1.y *= _sy;
	outline.p2.y *= _sy;
	outline.p3.y *= _sy;
	outline.p4.y *= _sy;
}

TranslateInst::TranslateInst ()
{
	this->x = 0.0;
	this->y = 0.0;
	this->duration = -1.0;
	this->delay = 0.0;
}

TranslateInst::TranslateInst (float x, float y, float duration, float delay)
{
	this->x = x;
	this->y = y;
	this->duration = duration;
	this->delay = delay;
}

void TranslateInst::set (float x, float y)
{
	this->x = x;
	this->y = y;
}

void TranslateInst::exec (Outline& outline, Position& position, float delta_t)
{
	//position.x += x;
	//position.y += y;

	glTranslatef (x, y, 0.0);
}

ScaleInst::ScaleInst ()
{
	this->sx = 1.0;
	this->sy = 1.0;
	this->duration = -1.0;
	this->delay = 0.0;
}

ScaleInst::ScaleInst (float sx, float sy, float duration, float delay)
{
	this->sx = sx;
	this->sy = sy;
	this->duration = duration;
	this->delay = delay;
}

void ScaleInst::set (float sx, float sy)
{
	this->sx = sx;
	this->sy = sy;
}

void ScaleInst::exec (Outline& outline, Position& position, float delta_t)
{
	outline.p1.x *= sx;
	outline.p2.x *= sx;
	outline.p3.x *= sx;
	outline.p4.x *= sx;

	outline.p1.y *= sy;
	outline.p2.y *= sy;
	outline.p3.y *= sy;
	outline.p4.y *= sy;
}
