#include <iostream>

#include "collisiondetector.h"
#include "testsquare.h"

using namespace std;

CollisionDetector::CollisionDetector (Player* player, float tile_size)
{
	this->player = player;
	this->tile_size = tile_size;
}

CollisionDetector::~CollisionDetector ()
{
}

void CollisionDetector::runAgainst (Tilemap& tilemap, WorldModifier& worldmodifier, float delta_t)
{	
	Tilemap::iterator row , start, end;
	FreeObjects::iterator object;

	int start_ = getLowerRow ();
	int end_ = getUpperRow ();

	if (start_ < 0) start_ = 0;
	if (end_ < 0) end_ = 0;
	if (start_ > (int)tilemap.size()) start_ = tilemap.size();
	if (end_ > (int)tilemap.size()) end_ = tilemap.size ();

	start = tilemap.begin () + start_;
	end = tilemap.begin () + end_;

	for (row = start; row != end; row++) {
		for (object = row->begin(); object != row->end (); object++) {
			if ((*object).second->getShape ()->check (player->getShape ()))
				(*object).second->getShape()->manipulate (player, worldmodifier, delta_t);
		}
	}

	worldmodifier.execute ();
}

void CollisionDetector::runAgainst (FreeObjects& freeobjects, WorldModifier& worldmodifier, float delta_t)
{
	FreeObjects::iterator object;

	for (object = freeobjects.begin (); object != freeobjects.end (); object++) {
		if ((*object).second->getShape ()->check (player->getShape ()))
			(*object).second->getShape()->manipulate (player, worldmodifier, delta_t);
	}

	worldmodifier.execute ();
}

int CollisionDetector::getLowerRow ()
{
	return (int)(player->getShape ()->gety () / tile_size - (player->getHeight () +1.0) / tile_size);
}

int CollisionDetector::getUpperRow ()
{
	return (int)(player->getShape ()->gety () / tile_size + (player->getHeight () +1.0) / tile_size);
}
