#include <iostream>
#include <cmath>

#include "fallingrock.h"
#include "renderutil.h"
#include "worldmodifier.h"

#define MAX_FALLSPEED -3.0
#define VELOCITY_CHANGERATE 4.0

using namespace std;

FallingRock::FallingRock (float x, float y, Rocktype type)
	:
	BaseObject (&rockshape),
	rockshape (this)
{
	vertical_velocity = 0.0;
	lifetime = 2.0;

	rockshape.x = x;
	rockshape.y = y;
	rockshape.sidelength = 0.2;

	this->type = type;
}

FallingRock::~FallingRock ()
{
}

void FallingRock::update (float delta_t, WorldModifier& worldmodifier)
{
	lifetime -= delta_t;

	if (lifetime < 0);
// 		worldmodifier.deleteFreeObject (getSignature ());

	/* Physics */
	vertical_velocity -= VELOCITY_CHANGERATE * delta_t;

	if (vertical_velocity < MAX_FALLSPEED)
		vertical_velocity = MAX_FALLSPEED;

	rockshape.y += vertical_velocity * delta_t;
}

void FallingRock::draw ()
{

	switch (type) {
		case easy:
			RenderUtil::renderSquare (rockshape.x,
											  rockshape.y,
											  rockshape.sidelength, "data/gfx/easyrock.png");
		break;

		case normal:
			RenderUtil::renderSquare (rockshape.x,
											  rockshape.y,
											  rockshape.sidelength, "data/gfx/normalrock.png");
		break;

		case hard:
			RenderUtil::renderSquare (rockshape.x,
											  rockshape.y,
											  rockshape.sidelength, "data/gfx/hardrock.png");
		break;

		case indestructible:
			RenderUtil::renderSquare (rockshape.x,
											  rockshape.y,
											  rockshape.sidelength, "data/gfx/rock.png");
		break;
	}
}

TiXmlNode& FallingRock::getXMLRepresentation ()
{
	TiXmlElement fallingrock_node ("fallingrock");

	return fallingrock_node;
}


void FallingRock::RockShape::manipulate (Player* player, WorldModifier& worldmodifier, float delta_t)
{
	/* nothing */
}
