#include <iostream>
#include <cmath>

#include "goal.h"
#include "renderutil.h"
#include "worldmodifier.h"

using namespace std;

Goal::Goal (float x, float y)
	:
	BaseObject (&goalshape),
	goalshape (this)
{
	goalshape.x = x;
	goalshape.y = y;
	goalshape.sidelength = 0.2;
}
Goal::Goal (TiXmlElement* goal_node)
	:
	BaseObject (&goalshape),
	goalshape (this)
{
	TiXmlElement* position_node = goal_node->FirstChildElement ("position");
	if (position_node) {
		position_node->QueryFloatAttribute ("x", &goalshape.x);
		position_node->QueryFloatAttribute ("y", &goalshape.y);
		position_node->QueryFloatAttribute ("sidelength", &goalshape.sidelength);
	}
}

Goal::~Goal ()
{
}

void Goal::draw ()
{
	RenderUtil::renderSquare (goalshape.x,
									  goalshape.y,
									  goalshape.sidelength, "data/gfx/goal.png");
}

TiXmlNode& Goal::getXMLRepresentation ()
{
	TiXmlElement* goal_node = new TiXmlElement ("goal");
	TiXmlElement position_node = ("position");

	position_node.SetDoubleAttribute ("x", goalshape.x);
	position_node.SetDoubleAttribute ("y", goalshape.y);
	position_node.SetDoubleAttribute ("sidelength", goalshape.sidelength);

	goal_node->InsertEndChild (position_node);

	return *goal_node;
}

void Goal::setPosition (float x, float y)
{
	goalshape.x = x;
	goalshape.y = y;
}

void Goal::GoalShape::manipulate (Player* player, WorldModifier& worldmodifier, float delta_t)
{
	player->goalReached ();
}
