#include <stdexcept>
#include <iostream>

#include "highscores.h"
#include "tinyxml/tinyxml.h"

using std::domain_error;
using std::cout;
using std::endl;

HighScores::HighScores (string highscorefile)
{
	this->highscorefile = highscorefile;
}

HighScores::~HighScores ()
{
	/* nothing for now */
}

bool HighScores::checkRecord (int score, string levelname)
{
	map<string, Record>::iterator record_iterator;
	record_iterator = highscores.find (levelname);

	if (score <= 0)
		return false;

	if (record_iterator != highscores.end ()) 
	{
		if (record_iterator->second.score < score)
			return true;
		else
			return false;
	}
	
	return true;	// no previous record
}

void HighScores::updateRecord (int score, string levelname, string nickname)
{
	map<string, Record>::iterator record_iterator;
	record_iterator = highscores.find (levelname);

	if (record_iterator != highscores.end ()) 
	{
		record_iterator->second.score = score;
		record_iterator->second.recordholder = nickname;
	}
	else 
	{
		Record record;

		record.score = score;
		record.levelname = levelname;
		record.recordholder  = nickname;

		highscores[levelname] = record;
	}
}

void HighScores::loadHighScores ()
{
	TiXmlDocument document (highscorefile.c_str ());
	TiXmlHandle handle (&document);
	
	if (!document.LoadFile ())
	{
		document.Clear ();
		throw domain_error ("Highscores: couldn't open " + highscorefile + ". If this is the first time you run superbunguru it's ok.");
	}

	TiXmlElement* record_node = handle.FirstChild("highscores").FirstChild("record").Element ();

	while (record_node != 0)
	{
		Record record;

		if (record_node->QueryIntAttribute ("score", &record.score) != TIXML_SUCCESS)
			throw domain_error ("malformed highscore file!");

		if (record_node->Attribute ("levelname") != NULL)
			record.levelname = record_node->Attribute ("levelname");
		else
			throw domain_error ("malformed highscore file!");

		if (record_node->Attribute ("recordholder") != NULL)
			record.recordholder = record_node->Attribute ("recordholder");
		else
			throw domain_error ("malformed highscore file!");

		highscores[record.levelname] = record;
		record_node = record_node->NextSiblingElement ();
	}
}

void HighScores::saveHighScores ()
{
	TiXmlDocument document (highscorefile.c_str ());

	TiXmlElement body_node ("highscores");

	map<string, Record>::iterator record_iterator;

	for (record_iterator = highscores.begin (); record_iterator != highscores.end (); record_iterator++) {
		TiXmlElement record_node ("record");

		record_node.SetAttribute ("score", record_iterator->second.score);
		record_node.SetAttribute ("levelname", record_iterator->second.levelname.c_str ());
		record_node.SetAttribute ("recordholder", record_iterator->second.recordholder.c_str ());

		body_node.InsertEndChild (record_node);
	}

	document.InsertEndChild (body_node);
	document.SaveFile ();
}
