#include <stdexcept>
#include <stdlib.h>
#include <GL/gl.h>

#include "meter.h"
#include "../renderutil.h"

using std::domain_error;
using namespace SimpleHud;

Meter::MeterFactory Meter::meter_factory;
	
Meter::Meter (TiXmlElement* widget_node)
	:
	Widget (widget_node)
{
	percent = 1.0;

	if (widget_node->Attribute ("meter-texture"))
		meter_texture = widget_node->Attribute ("meter-texture");
	else
		throw domain_error ("Meter: missing 'meter-texture' attribute");

	if (widget_node->Attribute ("frame-texture"))
		frame_texture = widget_node->Attribute ("frame-texture");
	else
		frame_texture.clear ();

	if (widget_node->QueryFloatAttribute ("x", &x) != TIXML_SUCCESS)
		throw domain_error ("Meter: missing 'x' attribute");

	if (widget_node->QueryFloatAttribute ("y", &y) != TIXML_SUCCESS)
		throw domain_error ("Meter: missing 'y' attribute");

	if (widget_node->QueryFloatAttribute ("width", &w) != TIXML_SUCCESS)
		throw domain_error ("Meter: missing 'width' attribute");

	if (widget_node->QueryFloatAttribute ("height", &h) != TIXML_SUCCESS)
		throw domain_error ("Meter: missing 'height' attribute");
}

Meter::~Meter ()
{
	/* nothing */
}

void Meter::draw ()
{
	if (!frame_texture.empty ())
		RenderUtil::renderRectangle (x, y, w/2.0, h/2.0, frame_texture.c_str ());

	float p = (percent - 0.5) * 2;

	float p1_x = x - w/2; float p1_y = y - h/2 * p;
	float p2_x = x - w/2; float p2_y = y + h/2;
	float p3_x = x + w/2; float p3_y = y + h/2;
	float p4_x = x + w/2; float p4_y = y - h/2 * p;

	glEnable (GL_TEXTURE_2D);

	glBindTexture (GL_TEXTURE_2D, ResourceHandler::instance ()->recvTexture (meter_texture.c_str ()));
	glBegin (GL_QUADS);
		glTexCoord2f (0.0, 0.0		 ); glVertex2f (p2_x, p2_y);
		glTexCoord2f (1.0, 0.0		 ); glVertex2f (p3_x, p3_y);
		glTexCoord2f (1.0, percent  ); glVertex2f (p4_x, p4_y);
		glTexCoord2f (0.0, percent  ); glVertex2f (p1_x, p1_y);
	glEnd ();

	glDisable (GL_TEXTURE_2D);
}

void Meter::handleMessage (float message)
{
	percent = message;

	if (percent > 1.0)
		percent = 1.0;
	
	if (percent < 0.0)
		percent = 0.0;
}
