#include <stdexcept>

#include "text.h"
#include "../renderutil.h"

using std::domain_error;
using namespace SimpleHud;

Text::TextFactory Text::text_factory;
	
Text::Text (TiXmlElement* widget_node)
	:
	Widget (widget_node)
{
	if (widget_node->Attribute ("text"))
		text = widget_node->Attribute ("text");
	else
		throw domain_error ("Text: missing 'text' attribute");

	if (widget_node->Attribute ("font"))
		font = widget_node->Attribute ("font");
	else
		throw domain_error ("Text: missing 'font' attribute");

	if (widget_node->Attribute ("alignment")) {
		if (strcmp (widget_node->Attribute ("alignment"), "left") == 0)
			text_alignment = RenderUtil::left;
		if (strcmp (widget_node->Attribute ("alignment"), "center") == 0)
			text_alignment = RenderUtil::center;
		if (strcmp (widget_node->Attribute ("alignment"), "right") == 0)
			text_alignment = RenderUtil::right;
	}
	else
		text_alignment = RenderUtil::center;

//	if (widget_node->Attribute ("cache")) {
//		if (strcmp (widget_node->Attribute ("cache"), "false") == 0)
//			cache_text = false;
//	} 
//	else
//		cache_text = true;

	if (widget_node->QueryFloatAttribute ("x", &x) != TIXML_SUCCESS)
		throw domain_error ("Text: missing 'x' attribute");

	if (widget_node->QueryFloatAttribute ("y", &y) != TIXML_SUCCESS)
		throw domain_error ("Text: missing 'y' attribute");

//	if (widget_node->QueryFloatAttribute ("size", &size) != TIXML_SUCCESS)
//		throw domain_error ("Text: missing 'size' attribute");
}

Text::~Text ()
{
	/* nothing */
}

void Text::draw ()
{
	if (!text.empty ())
		RenderUtil::renderBitmapString (x, y, text_alignment, font.c_str (), text.c_str ());
}

void Text::handleMessage (string message)
{
	text = message;
}
