#include <cmath>
#include <iostream>

#include "inputcontroller.h"

InputController::InputController ()
{
	inputstring = "<straver>";
	return_pressed = false;
	textmode = false;
	jumpkey = false;
	pause = false;
	rshift = false;
	jump = false;
	left = false;
	right = false;
	quit = false;
	wheel_down = false,
	wheel_up = false;
	left_click = false;
	right_click = false;
	mouse_moved = false;
	motion_x = 0.0;
	motion_y = 0.0;
	mouse_x = 0.0;
	mouse_y = 0.0;

	for (int i = 0; i < 10; i++)
		digits[i] = false;
	
	SDL_ShowCursor (SDL_DISABLE);
 	//SDL_WM_GrabInput (SDL_GRAB_ON);
	SDL_EnableUNICODE(1);
}

InputController::~InputController ()
{
	SDL_WM_GrabInput (SDL_GRAB_OFF);
}

void InputController::readInput ()
{

	if (textmode)
	{
		while (SDL_PollEvent (&event)){
			switch (event.type){
				case SDL_QUIT:
					quit =true;
					break;
				case SDL_KEYDOWN:
					if (event.key.keysym.sym == SDLK_ESCAPE)
						textmode = false;
					if (event.key.keysym.sym == SDLK_RETURN) {
						return_pressed = true;
						textmode = false;
						break;
					}

					if ((event.key.keysym.unicode & 0xFF80) == 0){
						char ch = event.key.keysym.unicode & 0x7F;
						inputstring.append (1, ch);
					}
					else
						// FIXME add åöä support here
					break;
			}
		}
	}
	else 
	{
		return_pressed = false;
		mouse_moved = false;
		wheel_down = false;
		wheel_up = false;
		left_click = false;
		right_click = false;

		while (SDL_PollEvent (&event)){
			switch (event.type){
				case SDL_QUIT:
					quit =true;
					break;
				case SDL_KEYDOWN:
					for (int i = 0; i < 10; i++) {
						if (event.key.keysym.sym == SDLK_0 + i)
							digits[i] = true;
					}

					if (event.key.keysym.sym == SDLK_ESCAPE)
						quit = true;
					if (event.key.keysym.sym == SDLK_SPACE)
						jump = true;
					if (event.key.keysym.sym == SDLK_LEFT)
						left = true;
					if (event.key.keysym.sym == SDLK_RIGHT)
						right = true;
					if (event.key.keysym.sym == SDLK_RSHIFT)
						rshift = true;
					if (event.key.keysym.sym == SDLK_p)
						pause = !pause;
					if (event.key.keysym.sym == SDLK_t)
						textmode = true;
					if (event.key.keysym.sym == SDLK_g)
						SDL_WM_GrabInput(SDL_GRAB_ON);
					if (event.key.keysym.sym == SDLK_h)
						SDL_WM_GrabInput(SDL_GRAB_OFF);
					break;
				case SDL_KEYUP:
					for (int i = 0; i < 10; i++) {
						if (event.key.keysym.sym == SDLK_0 + i)
							digits[i] = false;
					}

					if (event.key.keysym.sym == SDLK_LEFT)
						left = false;
					if (event.key.keysym.sym == SDLK_RIGHT)
						right = false;
					if (event.key.keysym.sym == SDLK_RSHIFT)
						rshift = false;
					if (event.key.keysym.sym == SDLK_SPACE)
						jump = false;
					break;
				case SDL_MOUSEMOTION:
					mouse_moved = true;
					motion_x = event.motion.xrel;
					motion_y = event.motion.yrel;
					mouse_x = event.motion.x;
					mouse_y = event.motion.y;
					break;

				case SDL_MOUSEBUTTONDOWN:
					if (event.button.button == SDL_BUTTON_WHEELDOWN)
						wheel_down = true;
					if (event.button.button == SDL_BUTTON_WHEELUP)
						wheel_up = true;
					if (event.button.button == SDL_BUTTON_LEFT)
						left_click = true;
					if (event.button.button == SDL_BUTTON_RIGHT)
						right_click = true;

//  				case SDL_MOUSEBUTTONUP:
//  					if (event.button.button == SDL_BUTTON_LEFT)
//  						left_click = false;
//  					if (event.button.button == SDL_BUTTON_RIGHT)
//  						right_click = false;
			}
		}
	}
}

bool InputController::digit (int d)
{
	return digits[d];
}

