#ifndef LEVEL_EDITOR_H
#define LEVEL_EDITOR_H

#include <vector>
#include <map>

#include "hud/hud.h"
#include "world.h"
#include "cursor.h"
#include "renderer.h"
#include "worldmodifier.h"
#include "objectfactory.h"
#include "inputcontroller.h"

using namespace std;

#define SELECT_ROCK 0
#define SELECT_ROCK_EASY 1
#define SELECT_ROCK_NORMAL 2
#define SELECT_ROCK_HARD 3
#define SELECT_GOAL 4
#define SELECT_POWERUP 5
#define SELECT_PLAYER 6

enum EditMode {
	freemode, tilemode, settingsmode
};

struct Settings {
	float startposition_x;
	float startposition_y;
};

class LevelEditor : public World {

	public:
	LevelEditor (const char *levelfile);
	~LevelEditor ();

	void edit ();
	void save (const char* levelfile);
	void load (const char* levelfile);

	private:

	void loadTemplate ();
	void switchObjectUp ();
	void switchObjectDown ();
	void addRows (int num_rows);
	void placeObject (float x, float y);
	void removeObject (float x, float y);
	float snapToGrid (float pos);

	vector<string> objectlist;

	float zoom;

	float viewport_x;
	float viewport_y;

	float cursor_x;
	float cursor_y;
	int select;

	Cursor cursor;
	EditMode editmode;
	Settings settings;

	InputController inputcontroller;
	WorldModifier worldmodifier;
	ObjectFactory objectfactory;
	Renderer renderer;
	Hud hud;
};


#endif //LEVEL_EDITOR_H
