#include <iostream>

#include "absoluteframe.h"
#include "menufactory.h"
#include "../renderutil.h"

using namespace SimpleMenu;

AbsoluteFrame::AbsoluteFrameFactory AbsoluteFrame::absolute_frame_factory;

AbsoluteFrame::AbsoluteFrame (TiXmlElement* menu_node, Frame* parent)
	:
	Frame (parent)
{
	// Cosmetic Stuff
	if (menu_node->Attribute ("backdrop") != NULL)
		backdrop = menu_node->Attribute ("backdrop");
	else
		backdrop.clear ();


	// Fill frame with content
	menu_node = menu_node->FirstChildElement ();

	while (menu_node) {
		content.push_back (dynamic_cast<Frame*>(Factory::create (menu_node, this)));
		menu_node = menu_node->NextSiblingElement ();
	}

	no_selectable_frames = true;
	for (selected_object = content.begin (); selected_object != content.end (); selected_object++) {
		if ((*selected_object)->isSelectable ())
			no_selectable_frames = false; 
	}

	if (!no_selectable_frames) {
		selected_object = --content.end ();
		selectNext ();
	}
}

AbsoluteFrame::~AbsoluteFrame ()
{
	list<Frame*>::iterator it;
	
	for (it = content.begin (); it != content.end (); it++) {
		delete (*it);
	}
}

void AbsoluteFrame::draw (float w, float h, float x, float y)
{
	if (!backdrop.empty ())
		drawBackdrop (w, h, x, y);

	float convert_w = w / 400;
	float convert_h = h / 300;
	float convert_x = w / 400;
	float convert_y = h / 300;

	for (list<Frame*>::iterator it = content.begin (); it != content.end (); it++) {
		float local_w = (*it)->getWidth () * convert_w;
		float local_h = (*it)->getHeight () * convert_h;
		float local_x = x + (*it)->getPositionX () * convert_x;
		float local_y = y + (*it)->getPositionY () * convert_y;

		(*it)->draw (local_w, local_h, local_x, local_y);
	}
}

void AbsoluteFrame::drawBackdrop (float w, float h, float x, float y)
{
	RenderUtil::renderRectangleTiled (x, y, w, h, 4.0, backdrop.c_str ());
}

void AbsoluteFrame::handleKeyPress (SDL_keysym key)
{
	(*selected_object)->handleKeyPress (key);
}

void AbsoluteFrame::selected ()
{
	/* do nothing */
}

void AbsoluteFrame::selectPrev ()
{
	if (no_selectable_frames)
		return;

	if (selected_object == content.begin ()) {
		selected_object = --content.end ();
	}
	else {
		selected_object--;
	}

	if (!(*selected_object)->isSelectable ())
		selectPrev ();

	(*selected_object)->selected ();
}

void AbsoluteFrame::selectNext ()
{
	if (no_selectable_frames)
		return;

	if (selected_object == --content.end ()) {
		selected_object = content.begin ();
	}
	else {
		selected_object++;
	}

	if (!(*selected_object)->isSelectable ())
		selectNext ();

	(*selected_object)->selected ();
}

void AbsoluteFrame::selectLeft ()
{
	/* nothing */
}

void AbsoluteFrame::selectRight ()
{
	/* nothing */
}
