#include <iostream>

#include "dynamicframe.h"
#include "menufactory.h"
#include "../renderutil.h"

using namespace std;
using namespace SimpleMenu;

DynamicFrame::DynamicFrameFactory DynamicFrame::dynamic_frame_factory;

DynamicFrame::DynamicFrame (TiXmlElement* menu_node, Frame* parent)
	:
	Frame (parent)
{
	menu_node->QueryFloatAttribute ("x", &position_x);
	menu_node->QueryFloatAttribute ("y", &position_y);
	menu_node->QueryFloatAttribute ("w", &width);
	menu_node->QueryFloatAttribute ("h", &height);

	if (menu_node->Attribute ("selectable") != NULL) {
		if (strcmp ("true", menu_node->Attribute ("selectable")) == 0)
			setSelectable (true);
		else if (strcmp ("false", menu_node->Attribute ("selectable")) == 0)
			setSelectable (false);
	}

	// Cosmetic Stuff
	if (menu_node->Attribute ("backdrop") != NULL)
		backdrop = menu_node->Attribute ("backdrop");
	else
		backdrop.clear ();


	// Grapical border stuff
	if (menu_node->QueryFloatAttribute ("border-upper", &border_upper) != TIXML_SUCCESS)
		border_upper = 0.0;

	if (menu_node->QueryFloatAttribute ("border-lower", &border_lower) != TIXML_SUCCESS)
		border_lower = 0.0;

	if (menu_node->QueryFloatAttribute ("border-right", &border_right) != TIXML_SUCCESS)
		border_right = 0.0;

	if (menu_node->QueryFloatAttribute ("border-left", &border_left) != TIXML_SUCCESS)
		border_left = 0.0;

	float holder;
	if (menu_node->QueryFloatAttribute ("border-horizontal", &holder) == TIXML_SUCCESS) {
		border_upper = holder;
		border_lower = holder;
	}

	if (menu_node->QueryFloatAttribute ("border-vertical", &holder) == TIXML_SUCCESS) {
		border_right = holder;
		border_left = holder;
	}

	if (menu_node->Attribute ("upper-horizontal-border-texture") != NULL)
		upper_horizontal_border_texture = menu_node->Attribute ("upper-horizontal-border-texture");
	else
		border_upper = 0.0;

	if (menu_node->Attribute ("lower-horizontal-border-texture") != NULL)
		lower_horizontal_border_texture = menu_node->Attribute ("lower-horizontal-border-texture");
	else
		border_lower = 0.0;

	if (menu_node->Attribute ("left-vertical-border-texture") != NULL)
		left_vertical_border_texture = menu_node->Attribute ("left-vertical-border-texture");
	else
		border_left = 0.0;

	if (menu_node->Attribute ("right-vertical-border-texture") != NULL)
		right_vertical_border_texture = menu_node->Attribute ("right-vertical-border-texture");
	else
		border_right = 0.0;

	// Create the stuff inside the frame
	menu_node = menu_node->FirstChildElement ();

	while (menu_node) {
		content.push_back (Factory::create (menu_node, this));
		menu_node = menu_node->NextSiblingElement ();
	}

	if (!content.empty ())
		selected_object = content.begin ();
	else
		selected_object = content.end ();
}

DynamicFrame::~DynamicFrame ()
{
	list<MenuObject*>::iterator it;
	
	for (it = content.begin (); it != content.end (); it++) {
		delete (*it);
	}
}

void DynamicFrame::draw (float w, float h, float x, float y)
{
	if (border_upper > 0.0 || border_lower > 0.0 || border_right > 0.0 || border_left > 0.0)
		drawBorder (w, h, x, y);

	float _w = w - border_right - border_left;
	float _h = h - border_upper - border_lower;
	float _x = x + border_left - border_right;
	float _y = y + border_upper - border_lower;

	if (!backdrop.empty ())
		drawBackdrop (_w, _h, _x, _y);

	drawContent (_w, _h, _x, _y);
}

void DynamicFrame::insert (MenuObject* menu_object)
{
	content.push_back (menu_object);
}

void DynamicFrame::reset ()
{
	selected_object = content.begin ();
}

void DynamicFrame::drawContent (float w, float h, float x, float y)
{
	float height = h / content.size ();
	float position_y = y - h / 2 + height / 2;

	for (list<MenuObject*>::iterator it = content.begin (); it != content.end (); it++) {
		(*it)->draw (w, height, x, position_y);
		position_y += height;
	}
}

void DynamicFrame::drawBorder (float w, float h, float x, float y)
{
	// left vertical
	RenderUtil::renderRectangle (x - w/2.0 + border_left/2.0, 
										  y + border_upper - border_lower, 
										  border_left/2.0, 
										  h/2.0 - border_upper/2.0 - border_lower/2.0, 
										  right_vertical_border_texture.c_str ());

	// right vertical
	RenderUtil::renderRectangle (x + w/2.0 - border_right/2.0, 
										  y + border_upper - border_lower, 
										  border_right/2.0, 
										  h/2.0 - border_upper/2.0 - border_lower/2.0, 
										  left_vertical_border_texture.c_str ());

	// lower horizontal
	RenderUtil::renderRectangle (x,
										  y + h/2.0 - border_upper/2.0, 
										  w/2.0, 
										  border_upper/2.0,
										  lower_horizontal_border_texture.c_str ());

	// upper horizontal
	RenderUtil::renderRectangle (x,
										  y - h/2.0 + border_lower/2.0, 
										  w/2.0, 
										  border_lower/2.0,
										  upper_horizontal_border_texture.c_str ());
}

void DynamicFrame::drawBackdrop (float w, float h, float x, float y)
{
	RenderUtil::renderRectangleTiled (x, y, w, h, 4.0, backdrop.c_str ());
}

void DynamicFrame::handleKeyPress (SDL_keysym key)
{
	if (selected_object != content.end ())
		(*selected_object)->handleKeyPress (key);
}

void DynamicFrame::selected ()
{
	if (selected_object != content.end ())
		(*selected_object)->selected ();
}

void DynamicFrame::selectPrev ()
{
	if (selected_object == content.begin ()) {
		selected_object = --content.end ();
	}
	else {
		selected_object--;
	}

	(*selected_object)->selected ();
}

void DynamicFrame::selectNext ()
{
	if (selected_object == --content.end ()) {
		selected_object = content.begin ();
	}
	else {
		selected_object++;
	}

	(*selected_object)->selected ();
}

void DynamicFrame::selectLeft ()
{
	/* nothing */
}

void DynamicFrame::selectRight ()
{
	/* nothing */
}
