#ifndef DYNAMIC_FRAME_H
#define DYNAMIC_FRAME_H

#include <list>
#include <iostream>
#include <SDL/SDL.h>

#include "frame.h"
#include "menufactory.h"
#include "../tinyxml/tinyxml.h"


using namespace std;

namespace SimpleMenu
{
	class DynamicFrame : public Frame
	{
		public:

		DynamicFrame (TiXmlElement* menu_node, Frame* parent);
		virtual ~DynamicFrame ();

		void draw (float w, float h, float x, float y);
		void handleKeyPress (SDL_keysym key);
		void insert (MenuObject* menu_object);
		void selected ();
		void reset ();
		void selectPrev ();
		void selectNext ();
		void selectLeft ();
		void selectRight ();

		protected:

		virtual void drawContent (float w, float h, float x, float y);
		virtual void drawBorder (float w, float h, float x, float y);
		void drawBackdrop (float w, float h, float x, float y);
	
		float border_upper;
		float border_lower;
		float border_right;
		float border_left;
		string left_vertical_border_texture;
		string right_vertical_border_texture;
		string upper_horizontal_border_texture;
		string lower_horizontal_border_texture;

		list<MenuObject*>::iterator selected_object;
		list<MenuObject*> content;

		private:

		class DynamicFrameFactory : public Factory
		{
			public:	

			DynamicFrameFactory ()
			:	Factory ("dynamic-frame", this) { }

			MenuObject* _create (TiXmlElement* menu_node, Frame* parent)
			{
				return new DynamicFrame (menu_node, parent);
			}
		};

		static DynamicFrameFactory dynamic_frame_factory;
		string backdrop;
	};
}

#endif // DYNAMIC_FRAME_H
