#include <dirent.h>
#include <iostream>
#include <stdexcept>

#include "fileview.h"
#include "textbutton.h"

using namespace std;
using namespace SimpleMenu;

FileView::FileViewFactory FileView::file_view_factory;

FileView::FileView (TiXmlElement* menu_node, Frame* parent)
	:
	ScrollFrame (menu_node, parent)
{
	if (menu_node->Attribute ("path") != NULL)
		directory_path = menu_node->Attribute("path");
	else
		throw domain_error ("FileView: missing 'path' attribute");
	
	string button_background_texture;
	string button_selected_texture;

	if (menu_node->Attribute ("button-background-texture") != NULL)
		button_background_texture = menu_node->Attribute ("button-background-texture");

	if (menu_node->Attribute ("button-selected-texture") != NULL)
		button_selected_texture = menu_node->Attribute ("button-selected-texture");

	if (directory_path.at (directory_path.length () -1) != '/')
		directory_path.append ("/");

	ResourceHandler* resource_handler = ResourceHandler::instance ();

	struct dirent **namelist;
	int n;

	n = scandir ((resource_handler->getDataPath ()+ directory_path).c_str (), &namelist, 0, alphasort);
	if (n < 0)
		cout << "error reading: " << directory_path << endl;
	else {
		while (n--) {
			TiXmlDocument levelinfo ((resource_handler->getDataPath () + directory_path + namelist[n]->d_name).c_str ());

			if (!levelinfo.LoadFile ()) {
				delete namelist[n];
				continue;
			}

//				throw domain_error ("LevelView: Could open unlocked level: " + levelfile);
			
			TiXmlHandle handle_levelinfo (&levelinfo);
			TiXmlElement* info_node = handle_levelinfo.FirstChild ("superbungurulevel").Element ();

			string levelname;
			if (info_node->Attribute ("name") != NULL)
				levelname = info_node->Attribute ("name");
			else
				levelname = "unnamed level (fix it!)";


			TiXmlElement textbutton_node ("textbutton");
			textbutton_node.SetAttribute ("id", "fileview");
			textbutton_node.SetAttribute ("size", 24);
			textbutton_node.SetAttribute ("text", levelname.c_str ());
			textbutton_node.SetAttribute ("alt-text", (directory_path + namelist[n]->d_name).c_str ());
			textbutton_node.SetAttribute ("background-texture", button_background_texture.c_str ());
			textbutton_node.SetAttribute ("selected-texture", button_selected_texture.c_str ());

			TextButton* textbutton = new TextButton (&textbutton_node, this);
			insert (textbutton);

			delete namelist[n];
		}
		delete namelist;
	}

	reset ();
}

FileView::~FileView ()
{
}
