#include <iostream>
#include <stdexcept>

#include "highscoreview.h"
#include "horizontaldynamicframe.h"
#include "textbutton.h"
#include "../resourcehandler.h"

using namespace std;
using namespace SimpleMenu;

HighScoreView::HighScoreViewFactory HighScoreView::highscore_view_factory;

HighScoreView::HighScoreView (TiXmlElement* menu_node, Frame* parent)
	:
	ScrollFrame (menu_node, parent)
{
	if (menu_node->Attribute ("highscore-list") != NULL)
		highscore_list = menu_node->Attribute("highscore-list");
	
	string button_background_texture;
	string button_selected_texture;

	if (menu_node->Attribute ("button-background-texture") != NULL)
		button_background_texture = menu_node->Attribute ("button-background-texture");

	if (menu_node->Attribute ("button-selected-texture") != NULL)
		button_selected_texture = menu_node->Attribute ("button-selected-texture");

	ResourceHandler* resource_handler = ResourceHandler::instance ();
	TiXmlDocument document((resource_handler->getDataPath () + highscore_list).c_str ());
	TiXmlHandle handle (&document);

	if (!document.LoadFile ())  {
		throw domain_error ("HighScoreView: Failed to open " + highscore_list);
	}

	TiXmlElement* record_node = handle.FirstChild ("highscores").FirstChild ("record").Element ();

	while (record_node)	{
		
		TiXmlElement horizontal_frame_node ("horizontal-dynamic-frame");
		HorizontalDynamicFrame* horizontal_frame = new HorizontalDynamicFrame (&horizontal_frame_node, this);

		string score = "N/A";
		string levelname = "N/A";
		string recordholder = "N/A";

		if (record_node->Attribute ("score") != NULL)
			score = record_node->Attribute ("score");

		if (record_node->Attribute ("levelname") != NULL)
			levelname = record_node->Attribute ("levelname");

		if (record_node->Attribute ("recordholder") != NULL)
			recordholder = record_node->Attribute ("recordholder");

		TiXmlElement textbutton_node ("textbutton");
		textbutton_node.SetAttribute ("id", "highscoreview");
		textbutton_node.SetAttribute ("size", 24);
		textbutton_node.SetDoubleAttribute ("padding", 5.0);
		textbutton_node.SetAttribute ("alignment", "left");
		textbutton_node.SetAttribute ("text", levelname.c_str ());
		textbutton_node.SetAttribute ("background-texture", button_background_texture.c_str ());
		textbutton_node.SetAttribute ("selected-texture", button_selected_texture.c_str ());

		horizontal_frame->insert (new TextButton (&textbutton_node, horizontal_frame));

		textbutton_node.SetAttribute ("text", score.c_str ());
		textbutton_node.SetAttribute ("alignment", "center");

		horizontal_frame->insert (new TextButton (&textbutton_node, horizontal_frame));

		textbutton_node.SetAttribute ("text", recordholder.c_str ());
		textbutton_node.SetAttribute ("alignment", "left");

		horizontal_frame->insert (new TextButton (&textbutton_node, horizontal_frame));

		horizontal_frame->reset ();
		insert (horizontal_frame);
		record_node = record_node->NextSiblingElement ();
	}

	reset ();
}

HighScoreView::~HighScoreView ()
{

}
