#include <iostream>
#include <stdexcept>
#include <ctype.h>

#include "frame.h"
#include "inputbutton.h"
#include "menusystem.h"
#include "../renderutil.h"

using namespace std;
using namespace SimpleMenu;

InputButton::InputButtonFactory InputButton::input_button_factory;

InputButton::InputButton (TiXmlElement* menu_node, Frame* parent)
	:
	TextButton (menu_node, parent)
{
	if (menu_node->QueryIntAttribute ("input-limit", &char_input_limit) != TIXML_SUCCESS)
		char_input_limit = 10000;

	inputstate = false;
}

InputButton::~InputButton ()
{
	/* nothing */
}

void InputButton::handleKeyPress (SDL_keysym key)
{
	switch (key.sym)
	{
		case SDLK_DOWN:
			if (!inputstate) {
				texture = background_texture;
				parent->selectNext ();
			}
		break;

		case SDLK_UP:
			if (!inputstate) {
				texture = background_texture;;
				parent->selectPrev ();
			}
		break;

		case SDLK_RETURN:
			if (inputstate) {
				inputstate = false;
				if (callback != 0) {
					callback->string_arg1 = text;
					callback->string_arg2.clear ();
					callback->execute ();
				}
			}
			else {
				inputstate = true;
				text = "";
				char_count = 0;
			}
		break;

		case SDLK_BACKSPACE:
			if (inputstate) {
				if (!text.empty ()) {
					text.erase (--text.end());
					char_count--;
				}
			}
		break;

		default:
			if (inputstate) {
				if (!(key.unicode & 0xff80) && isprint (key.unicode & 0x7f) && char_input_limit > char_count ){
					char ch = key.unicode & 0x7f;
					text.append (1, ch);
					char_count++;
				}
			}
		break;
	}
}

/*
void InputButton::selected ()
{
	texture = selected_texture;
}
*/
