#include <iostream>
#include <stdexcept>

#include "levelview.h"
#include "textbutton.h"
#include "../resourcehandler.h"

using namespace std;
using namespace SimpleMenu;

LevelView::LevelViewFactory LevelView::level_view_factory;

LevelView::LevelView (TiXmlElement* menu_node, Frame* parent)
	:
	ScrollFrame (menu_node, parent)
{
	if (menu_node->Attribute ("progress-file") != NULL)
		level_progress_file = menu_node->Attribute("progress-file");
	
	string button_background_texture;
	string button_selected_texture;

	if (menu_node->Attribute ("button-background-texture") != NULL)
		button_background_texture = menu_node->Attribute ("button-background-texture");

	if (menu_node->Attribute ("button-selected-texture") != NULL)
		button_selected_texture = menu_node->Attribute ("button-selected-texture");

	ResourceHandler* resource_handler = ResourceHandler::instance ();
	TiXmlDocument document((resource_handler->getDataPath () + level_progress_file).c_str ());
	TiXmlHandle handle (&document);

	if (!document.LoadFile ())  {
		throw domain_error ("LevelView: Failed to open " + level_progress_file);
	}

	TiXmlElement* level_node = handle.FirstChild ("unlocked").FirstChild ("level").Element ();

	while (level_node) {
		string levelname = "N/A";
		string levelfile = "N/A";

		if (level_node->Attribute ("levelfile") != NULL) {
			levelfile = level_node->Attribute ("levelfile");

			TiXmlDocument levelinfo ((resource_handler->getDataPath () + levelfile).c_str ());

			if (!levelinfo.LoadFile ())
				throw domain_error ("LevelView: Could open unlocked level: " + levelfile);
			
			TiXmlHandle handle_levelinfo (&levelinfo);

			TiXmlElement* info_node = handle_levelinfo.FirstChild ("superbungurulevel").Element ();

			if (info_node->Attribute ("name") != NULL)
				levelname = info_node->Attribute ("name");
			else
				levelname = "unnamed level (fix it!)";
		}

		TiXmlElement textbutton_node ("textbutton");
		textbutton_node.SetAttribute ("id", "levelview");
		textbutton_node.SetAttribute ("size", 24);
		textbutton_node.SetAttribute ("text", levelname.c_str ());
		textbutton_node.SetAttribute ("alt-text", levelfile.c_str ());
		textbutton_node.SetAttribute ("background-texture", button_background_texture.c_str ());
		textbutton_node.SetAttribute ("selected-texture", button_selected_texture.c_str ());

		TextButton* textbutton = new TextButton (&textbutton_node, this);
		insert (textbutton);

		level_node = level_node->NextSiblingElement ();
	}

	reset ();
}

LevelView::~LevelView ()
{

}
