#include <iostream>
#include <stdexcept>

#include "frame.h"
#include "textbutton.h"
#include "menusystem.h"
#include "../renderutil.h"

using namespace std;
using namespace SimpleMenu;

TextButton::TextButtonFactory TextButton::text_button_factory;

TextButton::TextButton (TiXmlElement* menu_node, Frame* parent)
	:
	Widget ()
{
	this->parent = parent;

	if (menu_node->Attribute("id") != NULL)
		this->id = menu_node->Attribute("id");
	else
		throw domain_error ("TextButton: Missing id property");

	if (menu_node->Attribute("text") != NULL)
		text = menu_node->Attribute("text");

	if (menu_node->Attribute("alt-text") != NULL)
		alt_text = menu_node->Attribute("alt-text");

	if (menu_node->Attribute("background-texture") != NULL)
		background_texture = menu_node->Attribute("background-texture");

	if (menu_node->Attribute("selected-texture") != NULL)
		selected_texture = menu_node->Attribute("selected-texture");

	if (menu_node->QueryIntAttribute ("size", &size) != TIXML_SUCCESS)
		throw domain_error ("TextButton: missing 'size' attribute");

	if (menu_node->QueryFloatAttribute ("padding", &padding) != TIXML_SUCCESS)
		padding = 0.0;

	if (menu_node->Attribute ("alignment")) {
		if (strcmp (menu_node->Attribute ("alignment"), "left") == 0)
			text_alignment = RenderUtil::left;
		if (strcmp (menu_node->Attribute ("alignment"), "center") == 0)
			text_alignment = RenderUtil::center;
		if (strcmp (menu_node->Attribute ("alignment"), "right") == 0)
			text_alignment = RenderUtil::right;
	}
	else
		text_alignment = RenderUtil::center;

	texture = background_texture;
}

TextButton::TextButton (string text, string background_texture, string selected_texture, string id, Frame* parent)
	:
	Widget ()
{
	this->parent = parent;
	this->id = id;
	this->text = text;
	this->selected_texture = selected_texture;
	this->background_texture = background_texture;

	texture = background_texture;

	MenuSystem::registerWidget (this);
}

TextButton::~TextButton ()
{
	/* nothing */
}

void TextButton::draw (float w, float h, float x, float y)
{
	RenderUtil::renderRectangle (x, y, w/2.0, h/2.0, texture.c_str ());

	float _x;

	switch (text_alignment) {
		case RenderUtil::left:
			_x = x - w/2.0 + padding;
		break;

		case RenderUtil::center:
			_x = x;
		break;

		case RenderUtil::right:
			_x = x + w/2.0 - padding;
		break;
	}

	if (!text.empty ())
		RenderUtil::renderTruetypeString (_x, y, size, text_alignment, true, text.c_str ());
}

void TextButton::handleKeyPress (SDL_keysym key)
{
	switch (key.sym)
	{
		case SDLK_DOWN:
			texture = background_texture;
			parent->selectNext ();
		break;

		case SDLK_UP:
			texture = background_texture;;
			parent->selectPrev ();
		break;

		case SDLK_LEFT:
			texture = background_texture;;
			parent->selectLeft ();
		break;

		case SDLK_RIGHT:
			texture = background_texture;;
			parent->selectRight ();
		break;

		case SDLK_RETURN:
			if (callback != 0) {
				callback->string_arg1 = text;
				callback->string_arg2 = alt_text;
				callback->execute ();
			}
		break;

		default:
			// meeh
		break;
	}
}

void TextButton::selected ()
{
	texture = selected_texture;
}
