#include <stdexcept>

#include "objectfactory.h"
#include "powerup.h"
#include "rock.h"
#include "goal.h"

ObjectFactory::ObjectFactory ()
{
	lookup[string("rock")] = ROCK;
	lookup[string("easyrock")] = EASYROCK;
	lookup[string("normalrock")] = NORMALROCK;
	lookup[string("hardrock")] = HARDROCK;
	lookup[string("goal")] = GOAL;
	lookup[string("powerup")] = POWERUP;
}
ObjectFactory::~ObjectFactory ()
{
}

BaseObject* ObjectFactory::createObject (string name, float x, float y)
{
	map<string, ObjectType>::iterator it;
	it = lookup.find (name);

	if (it == lookup.end ()) {
		throw domain_error ("ObjectFactory: can't create a '" + name + "' object");
	}

	switch (it->second)
	{
		case ROCK:
			return new Rock (x,y, indestructible);
			break;

		case EASYROCK:
			return new Rock (x,y, easy);
			break;

		case NORMALROCK:
			return new Rock (x,y, normal);
			break;

		case HARDROCK:
			return new Rock (x,y, hard);
			break;

		case GOAL:
			return new Goal (x,y);
			break;

		case POWERUP:
			return new  PowerUp (x,y);
			break;

		default:
			throw domain_error ("ObjectFactory: I'm incorrectly defined (check the code!)");
	}

	return NULL; // shouldn't happen
}
BaseObject* ObjectFactory::createObject (string name, TiXmlElement* object_node)
{
	map<string, ObjectType>::iterator it;
	it = lookup.find (name);

	if (it == lookup.end ()) {
		throw domain_error ("ObjectFactory: can't create a '" + name + "' object");
	}

	switch (it->second)
	{
		case ROCK:
			return new Rock (object_node);
			break;

		case GOAL:
			return new Goal (object_node);
			break;

		case POWERUP:
			return new PowerUp (object_node);
			break;

		default:
			throw domain_error ("ObjectFactory: I'm incorrectly defined (check the code!)");
	}

	return NULL; // shouldn't happen
}
