#include <stdexcept>

#include "openglwindow.h"

using namespace std;

/****************************
 * Constructors/Destructors *
 ****************************/
 
/**
 * Empty Constructor
 */
OpenGLWindow::OpenGLWindow ()
{
	intialized = false;
	
	if (SDL_Init (SDL_INIT_VIDEO) < 0)
		throw domain_error ("OpenGLWindow: SDL initialization failed");
	else
		intialized = true;
}

/**
 * Destructor
 */
OpenGLWindow::~ OpenGLWindow()
{
	SDL_Quit ();
}

/***********
 * Methods *
 ***********/
 
/**
 * Create a opengl window
 */
void OpenGLWindow::setResolution (int width, int height, int depth, Uint32 flags) 
{
	if (!intialized)
		throw domain_error ("OpenGLWindow: Can't create a window if SDL isn't loaded");
	
   SDL_GL_SetAttribute (SDL_GL_DOUBLEBUFFER, 1);
   SDL_GL_SetAttribute (SDL_GL_RED_SIZE, 8);
   SDL_GL_SetAttribute (SDL_GL_GREEN_SIZE, 8);
   SDL_GL_SetAttribute (SDL_GL_BLUE_SIZE, 8);
   SDL_GL_SetAttribute (SDL_GL_ALPHA_SIZE, 8);
   SDL_GL_SetAttribute (SDL_GL_BUFFER_SIZE, 32);
 
	display_buffer = SDL_SetVideoMode (width, height, depth, flags);
	
	if (display_buffer == NULL)
		throw domain_error ("OpenGLWindow: Failed to set video mode");
	
	// init OpenGL
	glClearColor (0.0f, 0.0f, 0.0f, 0.0f); //black
	
	glViewport (0, 0, (GLsizei)width, (GLsizei)height);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	gluPerspective (90.0f, (GLdouble)width / (GLdouble)height, 0.0f, 10.0f);
	
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity ();
}

/**
 * switches to fullscreen if windowed or reverse
 */
void OpenGLWindow::toggleFullscreen () 
{
	if (display_buffer != NULL)
		SDL_WM_ToggleFullScreen (display_buffer);
}

/**
 * sets the caption of the window title
 */
void OpenGLWindow::setCaption (const char* caption) 
{
	SDL_WM_SetCaption (caption, NULL);
}
