#include <iostream>
#include <cmath>

#include "particlesystem.h"
#include "renderutil.h"

ParticleSystem::ParticleSystem (string texture_path,
										  float x, 
										  float y,
										  float size,
										  int num,
										  float rate,
										  float spread, 
										  float direction,
										  float max_speed,
										  float speed_variation,
										  float max_life,
										  float life_variation,
										  float base,
										  float xgravity,
										  float ygravity)
	:
	BaseObject (&particlesystemshape),
	particlesystemshape (this),
	is_active (true),
	timer (0.0)
{
	particlesystemshape.x = x;
	particlesystemshape.y = y;

	this->texture_path = texture_path;

	gx = xgravity;
	gy = ygravity;

	num_particles = num;

	this->size = size;
	this->rate = rate;
	this->spread = spread;
	this->direction = direction;
	this->max_speed = max_speed;
	this->speed_variation = speed_variation;
	this->max_life = max_life;
	this->life_variation = life_variation;
	this->base = base;
	

	if (rate <= 0) {
		while (num_particles > 0) {	
			spawnParticle ();
		}
	} 
}

ParticleSystem::ParticleSystem (TiXmlElement* psystem_node)
	:
	BaseObject (&particlesystemshape),
	particlesystemshape (this)
{

}

void ParticleSystem::setPosition (float x, float y)
{
	particlesystemshape.x = x;
	particlesystemshape.y = y;
}

void ParticleSystem::draw ()
{
	//if (!is_active)
	//	return;

	vector<ParticleShape>::iterator  particle;
	for (particle = particles.begin (); particle != particles.end (); particle++) {
		RenderUtil::renderSquareTrans (particle->x,
										  		 particle->y,
												 particle->sidelength,
												 particle->opacity, texture_path.c_str ());
	}
}

void ParticleSystem::update (float delta_t, WorldModifier& worldmodifier)
{
	//if (!is_active)
	//	return;

	vector<ParticleShape>::iterator  particle;
	for (particle = particles.begin (); particle != particles.end (); particle++) {
		particle->sx += gx * delta_t;
		particle->sy += gy * delta_t;

		particle->x += particle->sx * delta_t;
		particle->y += particle->sy * delta_t;

		particle->life -= delta_t;

		if (particle->life < 0.2) {
			particle->opacity = particle->life/0.2;
		}
	}

	particle = particles.begin ();
	while (particle != particles.end ()) {
		if (particle->life <= 0.0)
			particle = particles.erase (particle);
		else
			particle++;
	}

	if (is_active) {
		timer += delta_t;

		while (rate <= timer && num_particles > 0) {
			spawnParticle ();
			timer -= rate;
		}
	}

	if (is_active) {
		if (particles.empty  () && num_particles <= 0.0)
			worldmodifier.deleteObject (getSignature ());
	}
}

void ParticleSystem::spawnParticle ()
{
		ParticleShape particle;

		float random_number = drand48 () -0.5;

		particle.x = sin (direction) * random_number * base + particlesystemshape.x;
		particle.y = cos (direction) * random_number * base + particlesystemshape.y;
		particle.sidelength = size;
		particle.life = max_life - drand48 () * life_variation;
		particle.opacity = 1.0;

		particle.sx = cos (direction + spread * random_number) * (max_speed - drand48() * speed_variation);
		particle.sy = sin (direction + spread * random_number) * (max_speed - drand48() * speed_variation);

		particles.push_back (particle);

		num_particles--;
}

void ParticleSystem::spawnDummyParticle ()
{
	ParticleShape particle;

	particle.sidelength = 0.0;
	particle.life = drand48 () * life_variation;

	particles.push_back (particle);
}

TiXmlNode& ParticleSystem::getXMLRepresentation ()
{
	TiXmlElement psystem_node ("particlesystem");

	return psystem_node;
}
