#ifndef PARTICLE_SYSTEM_H
#define PARTICLE_SYSTEM_H

#include <vector>

#include "worldmodifier.h"
#include "baseobject.h"
#include "square.h"

using namespace std;

class ParticleSystem : public BaseObject
{
	public:
	ParticleSystem (string texture_path,			// particle texture
						 float x, 							// center of particle system
						 float y,							// center of particle system
						 float size,						// size of particles
						 int num,							// number of particles to be produced
						 float rate,						// rate of production (particles per second)
						 float spread, 					// spread angle
						 float direction,					// angle which the system is facing
						 float max_speed,					// max initial speed for a particle
						 float speed_variation,			// varation in speed for particles
						 float max_life = 5.0,			// max life time for a particle
						 float life_variation = 0.0,	// variation in life time for particles
						 float base = 0.0,				// spawn base (length of base which the particle can appear from)
						 float xgravity = 0.0,			// gravity in the x axis
						 float ygravity = -2.0);		// gravity in the y axis
	ParticleSystem (TiXmlElement* psystem_node);

	void setPosition (float x, float y);
	void active (bool set) { is_active = set; }
	void shutdown () { num_particles = 0; }
	virtual void draw ();
	virtual void update (float delta_t, WorldModifier& worldmodifier);
	virtual TiXmlNode& getXMLRepresentation ();

	private:

	class ParticleSystemShape;
	friend class ParticleSystemShape;

	class ParticleShape : public Square {
		public:

		void manipulate (Player* player, WorldModifier& worldmodifier, float delta_t) { /* empty */ }

		float opacity;
		float life;
		float sx, sy;
	};

	class ParticleSystemShape : public Shape {
		public:

		ParticleSystemShape (ParticleSystem* psystem_p) { this->psystem_p = psystem_p; }
		
		bool check (Shape* shape) { return false; }
		bool check (Square* square) { return false; }
		bool check (Circle* circle) { return false; }
		bool check (Shapeless* shapless) { return false; }
		void manipulate (Player* player, WorldModifier& worldmodifier, float delta_t) { /* empty */}

		float getx () { return x; }
		float gety () { return y; }

		float x, y;
		float radius;	// estimation of how large the system will be

		private:

		ParticleSystem* psystem_p;
	}particlesystemshape;

	void spawnParticle ();
	void spawnDummyParticle ();

	vector<ParticleShape> particles;

	bool is_active;
	float timer;

	string texture_path;

	float gx, gy;
	float size;
	int num_particles;	
//	int max_particles;	
	float rate;
	float spread;		
	float direction; 
	float base;
	float max_speed;
	float speed_variation;
	float max_life;
	float life_variation;
};

#endif // PARTICLE_SYSTEM_H
