#ifndef PLAYER_H
#define PLAYER_H

#include "baseobject.h"
#include "square.h"
#include "animation.h"
#include "basicanimations.h"
#include "textureanimation.h"
#include "worldmodifier.h"

enum Modifier {
	mod_normal, 
	mod_powerjump
};

enum DrawState {
	draw_standing,
	draw_falling,
	draw_jump,
	draw_powerjump
};

class PlayerShape : public Square
{
	public:
	void manipulate (Player* player, WorldModifier& worldmodifer, float delta_t) { /*this is a singleplayer game*/}

	float vertical_velocity;
};

class Player : public BaseObject
{
	public:
	
	Player ();
	~Player () { }

	virtual void draw ();
	virtual TiXmlNode& getXMLRepresentation ();
	void rechargePower (float delta_t);
	void reloadPower ();
	void collectPowerUp ();
	void goalReached ();
	bool isGoalReached ();
	void jump (float delta_t);
	void move_left (float delta_t);
	void move_right (float delta_t);
	virtual void update (float delta_t, WorldModifier& worldmodifier);
	int getNumPowerUps () { return collected_powerups; }
	float getHeight () { return playershape.sidelength; }
	PlayerShape& getPlayerShape () { return playershape; }

	private:

	int collected_powerups_combo;
	int collected_powerups;
	float total_jump_power;
	float jump_power;
	float combo_time;
	bool goal_reached;
	Modifier modifier;
	DrawState drawstate;

	// drawstuff
	PlayerShape playershape;
	TextureAnimation playeranimation;
	SimpleAnimation beam_animation;
	
	float powerjump_start;

	// Animations
	TranslateInst translate;
	ScaleInst scale;
	Rotate rotate;
};


#endif //PLAYER_H
