#include <stdexcept>
#include <iostream>

#include "progress.h"
#include "tinyxml/tinyxml.h"

using std::domain_error;
using std::cout;
using std::endl;

Progress::Progress (string progress_file)
{
	this->progress_file = progress_file;

	loadProgress ();
}

Progress::~Progress ()
{
	// nothing
}

bool Progress::update (string level)
{
	vector<LevelInfo>::iterator level_iterator;

	for (level_iterator = unlocked_levels.begin (); level_iterator != unlocked_levels.end (); level_iterator++) {
		if (level.compare (level_iterator->name) == 0 && level_iterator->completed == false) {
			level_iterator->completed = true;
			unlockLevel ();
			saveProgress ();
			return true;
		}
	}

	return false;
}

void Progress::unlockLevel ()
{
	if (locked_levels.empty ())
		return;

	LevelInfo& levelinfo = locked_levels.front ();
	unlocked_levels.push_back (levelinfo);
	locked_levels.erase (locked_levels.begin ());
}

void Progress::loadProgress ()
{
	TiXmlDocument document (progress_file.c_str ());
	TiXmlHandle handle (&document);
	
	if (!document.LoadFile ())
	{
		document.Clear ();
		throw domain_error ("couldn't open progress file!");
	}

	TiXmlElement* level_node = handle.FirstChild("unlocked").FirstChild("level").Element ();

	while (level_node) {
		LevelInfo levelinfo;

		if (level_node->Attribute ("levelfile") != NULL)
			levelinfo.name = level_node->Attribute ("levelfile");
		else
			throw domain_error ("Progress: Malformed progress file!");

		if (level_node->Attribute ("completed") != NULL) {
			if (string("true").compare (level_node->Attribute ("completed")) == 0)
				levelinfo.completed = true;
			else
				levelinfo.completed = false;
		}

		unlocked_levels.push_back (levelinfo);
		level_node = level_node->NextSiblingElement ();
	}

	level_node = handle.FirstChild("locked").FirstChild("level").Element ();

	while (level_node) {
		LevelInfo levelinfo;

		if (level_node->Attribute ("levelfile") != NULL)
			levelinfo.name = level_node->Attribute ("levelfile");
		else
			throw domain_error ("Progress: Malformed progress file!");

		levelinfo.completed = false;

		locked_levels.push_back (levelinfo);
		level_node = level_node->NextSiblingElement ();
	}
}

void Progress::saveProgress ()
{
	TiXmlDocument document (progress_file.c_str ());
	
	TiXmlElement unlocked_node ("unlocked");
	TiXmlElement locked_node ("locked");

	vector<LevelInfo>::iterator level_iterator;

	for (level_iterator = unlocked_levels.begin (); level_iterator != unlocked_levels.end (); level_iterator++) {
		TiXmlElement level_node ("level");

		level_node.SetAttribute ("levelfile", level_iterator->name.c_str ());

		if (level_iterator->completed)
			level_node.SetAttribute ("completed", "true");
		else
			level_node.SetAttribute ("completed", "false");
		
		unlocked_node.InsertEndChild (level_node);
	}

	for (level_iterator = locked_levels.begin (); level_iterator != locked_levels.end (); level_iterator++) {
		TiXmlElement level_node ("level");

		level_node.SetAttribute ("levelfile", level_iterator->name.c_str ());

		if (level_iterator->completed)
			level_node.SetAttribute ("completed", "true");
		else
			level_node.SetAttribute ("completed", "false");
		
		locked_node.InsertEndChild (level_node);
	}

	document.InsertEndChild (unlocked_node);
	document.InsertEndChild (locked_node);

	if (!document.SaveFile ())
		cout << "Progress: failed to save progress(" << progress_file << ")" << endl;
}
