#include <GL/gl.h>
#include <iostream>

#include "renderer.h"
#include "shape.h"
#include "resourcehandler.h"

#define VIEW_ZOOM_RATIO 1.41421

using namespace std;

Renderer::Renderer ()
{
	glEnable (GL_BLEND);
	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	renderhook_set = false;
	viewport_x = 0.0;
	viewport_y = 0.0;
	viewport_height = 1.4142;
}

Renderer::Renderer (Shape* shape)
{
	view_hook = shape;
	renderhook_set = true;

	zoom = 1.0;
	viewport_x = shape->getx ();
	viewport_y = shape->gety ();
	viewport_height = 1.4142;
}

Renderer::~Renderer ()
{
}

void Renderer::render (Tilemap& tilemap, float tile_size)
{
	Tilemap::iterator row, start, end;
	FreeObjects objectlist;

// 	float num_rows = tilemap.size ();
	
	int draw_begin = (int)(viewport_y / tile_size - viewport_height / tile_size);
	int draw_end = (int)(viewport_y / tile_size + viewport_height / tile_size);

	if (draw_end > (int)tilemap.size ())
		draw_end = tilemap.size ();

	if (draw_begin > (int)tilemap.size ())
		draw_begin = tilemap.size ();

	if (draw_begin < 0) draw_begin = 0;
	if (draw_end < 0) draw_end = 0;
	
	start = tilemap.begin () + draw_begin;
	end = tilemap.begin () + draw_end;

	for (row = start; row != end; row++) {
		objectlist.insert (row->begin (), row->end ());
	}
	
	render (objectlist);
}

void Renderer::render (FreeObjects& baseobjects)
{
	/* FIXME draw something fancy here */

	if (renderhook_set) {
		viewport_x = view_hook->getx ();

		if (viewport_y < view_hook->gety ())
			viewport_y = view_hook->gety ();
	}

// 	vector<BaseObject*>::iterator object;
	FreeObjects::iterator object;
	for (object = baseobjects.begin (); object != baseobjects.end (); ++object) {
		glLoadIdentity ();
		glTranslated (-viewport_x, -viewport_y, -(1.0 * zoom));

		(*object).second->draw ();
	}
}

void Renderer::renderHook (Shape* shape)
{
	view_hook = shape;
	renderhook_set = true;
}

void Renderer::setViewport (float x, float y, float zoom)
{
	viewport_x = x;
	viewport_y = y;
	this->zoom = zoom;

 	viewport_height =  zoom * VIEW_ZOOM_RATIO;
}

void Renderer::setZoom (float zoom)
{
	this->zoom = zoom;

	viewport_height = zoom * VIEW_ZOOM_RATIO;
}

void Renderer::getViewport (float* x, float* y, float* zoom)
{
	if (x != NULL)
		*x = viewport_x;

	if (y != NULL) 
		*y = viewport_y;

	if (zoom != NULL)
		*zoom = this->zoom;
}


/*
vector<BaseObject*> Game::tilesInView (Tilemap& tilemap)
{
	Tilemap::iterator row, start, end;
	FreeObjects objectlist;

	start = tilemap.begin () + (int)viewport_x;
	end = tilemap.begin () +  (int)viewport_x + 4;

	for (row = start; row != end; row++) {
		objectlist.insert (objectlist.end (), row->begin (), row->end ());
	}

	return objectlist;
}
*/


void Renderer::drawBackdrop ()
{
   ResourceHandler* resource_handler = ResourceHandler::instance ();
   GLuint backdrop = resource_handler->recvTexture ("data/gfx/bg03.png");

   glEnable (GL_TEXTURE_2D);
   //glDisable (GL_DEPTH_TEST);
	glLoadIdentity ();

   float x = viewport_x;
   float y = viewport_y;

   glColor3f (1.0, 1.0, 1.0);
   glBindTexture (GL_TEXTURE_2D, backdrop);
   glBegin (GL_QUADS);
   glTexCoord2f (-1.0 * zoom + x*1.0, -1.0 * zoom + y*1.0); glVertex3f (-1.4, -1.4, -(1.0));
   glTexCoord2f (-1.0 * zoom + x*1.0, +1.0 * zoom + y*1.0); glVertex3f (-1.4, +1.4, -(1.0));
   glTexCoord2f (+1.0 * zoom + x*1.0, +1.0 * zoom + y*1.0); glVertex3f (+1.4, +1.4, -(1.0));
   glTexCoord2f (+1.0 * zoom + x*1.0, -1.0 * zoom + y*1.0); glVertex3f (+1.4, -1.4, -(1.0));
   glEnd ();
}
