#include <cmath>
#include <iostream>
#include <stdexcept>

#include "renderutil.h"
#include "resourcehandler.h"
#include "bitmapfont.h"

map<string, RenderUtil::TextTexture> RenderUtil::text_cache;

void RenderUtil::renderSquare(float x,
										float y,
										float side,
										const char* texture_path)
{
		ResourceHandler* resource_handler = ResourceHandler::instance ();
		GLuint texture = resource_handler->recvTexture (texture_path);

		glEnable (GL_TEXTURE_2D);
		glColor4f (1.0, 1.0, 1.0, 1.0);
		glBindTexture (GL_TEXTURE_2D, texture);

		float p1_x = x - side; float p1_y = y - side;
		float p2_x = x - side; float p2_y = y + side;
		float p3_x = x + side; float p3_y = y + side;
		float p4_x = x + side; float p4_y = y - side;

      glBegin (GL_QUADS);
      	glTexCoord2f (0.0, 0.0); glVertex3f (p2_x, p2_y, 0.0);
      	glTexCoord2f (1.0, 0.0); glVertex3f (p3_x, p3_y, 0.0);
      	glTexCoord2f (1.0, 1.0); glVertex3f (p4_x, p4_y, 0.0);
      	glTexCoord2f (0.0, 1.0); glVertex3f (p1_x, p1_y, 0.0);
      glEnd ();

		glDisable (GL_TEXTURE_2D);
}

void RenderUtil::renderSquareTrans (float x,
											 	float y, 
											 	float side,
											 	float alpha,
											 	const char* texture_path)
{
		ResourceHandler* resource_handler = ResourceHandler::instance ();
		GLuint texture = resource_handler->recvTexture (texture_path);

		glEnable (GL_TEXTURE_2D);
		glColor4f (1.0, 1.0, 1.0, alpha);
		glBindTexture (GL_TEXTURE_2D, texture);

		float p1_x = x - side; float p1_y = y - side;
		float p2_x = x - side; float p2_y = y + side;
		float p3_x = x + side; float p3_y = y + side;
		float p4_x = x + side; float p4_y = y - side;

      glBegin (GL_QUADS);
      	glTexCoord2f (0.0, 0.0); glVertex3f (p2_x, p2_y, 0.0);
      	glTexCoord2f (1.0, 0.0); glVertex3f (p3_x, p3_y, 0.0);
      	glTexCoord2f (1.0, 1.0); glVertex3f (p4_x, p4_y, 0.0);
      	glTexCoord2f (0.0, 1.0); glVertex3f (p1_x, p1_y, 0.0);
      glEnd ();

		glDisable (GL_TEXTURE_2D);
}

void RenderUtil::renderCircle (float x,
										 float y,
										 float radius,
										 int detail,
										 const char* texture_path)
{
	ResourceHandler* resource_handler = ResourceHandler::instance ();
	GLuint texture = resource_handler->recvTexture (texture_path);

 	GLfloat coeff_s[] = {1.0, 0.0, 0.0, 0.0};
 	GLfloat coeff_t[] = {0.0, 1.0, 0.0, 0.0};
 	glTexGeni (GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR); // FIXME change GL_EYE_LINEAR
 	glTexGenfv (GL_S, GL_EYE_PLANE, coeff_s);	// FIXME change GL_EYE_PLANE

 	glTexGeni (GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR); // FIXME change GL_EYE_LINEAR
 	glTexGenfv (GL_T, GL_EYE_PLANE, coeff_t); // FIXME change GL_EYE_PLANE

 	glEnable (GL_TEXTURE_GEN_S);
	glEnable (GL_TEXTURE_GEN_T);
	glEnable (GL_TEXTURE_2D);
	glColor4f (1.0, 1.0, 1.0, 1.0);
	glBindTexture (GL_TEXTURE_2D, texture);

	float step = 2 * M_PI / detail;

	glBegin (GL_POLYGON);
		for (float i = 0; i <= M_PI*2; i += step) {
			glVertex3f (x + cos(i) * radius, y + sin(i) * radius, 0.0f);
		}
	glEnd();

	glDisable (GL_TEXTURE_GEN_S);
	glDisable (GL_TEXTURE_GEN_T);
	glDisable (GL_TEXTURE_2D);
}

void RenderUtil::renderRectangle (float x,
											 float y,
										  	 float width,
										  	 float height,
										  	 const char *texture_path)
{
		ResourceHandler* resource_handler = ResourceHandler::instance ();
		GLuint texture = resource_handler->recvTexture (texture_path);

		glEnable (GL_TEXTURE_2D);
		glColor4f (1.0, 1.0, 1.0, 1.0);
		glBindTexture (GL_TEXTURE_2D, texture);

		float p1_x = x - width; float p1_y = y - height;
		float p2_x = x - width; float p2_y = y + height;
		float p3_x = x + width; float p3_y = y + height;
		float p4_x = x + width; float p4_y = y - height;

      glBegin (GL_QUADS);
      	glTexCoord2f (0.0, 1.0); glVertex3f (p2_x, p2_y, 0.0);
      	glTexCoord2f (1.0, 1.0); glVertex3f (p3_x, p3_y, 0.0);
      	glTexCoord2f (1.0, 0.0); glVertex3f (p4_x, p4_y, 0.0);
      	glTexCoord2f (0.0, 0.0); glVertex3f (p1_x, p1_y, 0.0);
      glEnd ();

		glDisable (GL_TEXTURE_2D);
}

void RenderUtil::renderRectangle (float x,
											 float y,
										  	 float width,
										  	 float height,
										  	 GLuint texture)
{
		glEnable (GL_TEXTURE_2D);
		glColor4f (1.0, 1.0, 1.0, 1.0);
		glBindTexture (GL_TEXTURE_2D, texture);

		float p1_x = x - width; float p1_y = y - height;
		float p2_x = x - width; float p2_y = y + height;
		float p3_x = x + width; float p3_y = y + height;
		float p4_x = x + width; float p4_y = y - height;

      glBegin (GL_QUADS);
      	glTexCoord2f (0.0, 0.0); glVertex3f (p2_x, p2_y, 0.0);
      	glTexCoord2f (1.0, 0.0); glVertex3f (p3_x, p3_y, 0.0);
      	glTexCoord2f (1.0, 1.0); glVertex3f (p4_x, p4_y, 0.0);
      	glTexCoord2f (0.0, 1.0); glVertex3f (p1_x, p1_y, 0.0);
      glEnd ();

		glDisable (GL_TEXTURE_2D);
}

void RenderUtil::renderRectangleTiled (float x,
													float y,
													float width,
													float height,
													float tiles,
													const char* texture_path)
{
	ResourceHandler* resource_handler = ResourceHandler::instance ();
	GLuint texture = resource_handler->recvTexture (texture_path);

	/*
	GLfloat coeff_s[] = {1.0, 0.0, 0.0, 0.0};
	GLfloat coeff_t[] = {0.0, 1.0, 0.0, 0.0};
	glTexGeni (GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
	glTexGenfv (GL_S, GL_EYE_PLANE, coeff_s);

	glTexGeni (GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
	glTexGenfv (GL_T, GL_EYE_PLANE, coeff_t);

	glEnable (GL_TEXTURE_GEN_S);
	glEnable (GL_TEXTURE_GEN_T);
	*/
	glEnable (GL_TEXTURE_2D);

	glColor4f (1.0, 1.0, 1.0, 1.0);
	glBindTexture (GL_TEXTURE_2D, texture);

	width = width / 2.0;
	height = height / 2.0;

	float tc_x = (height / width) * tiles;
	float tc_y = (width / height) * tiles * 3/7;

	float p1_x = x - width; float p1_y = y - height;
	float p2_x = x - width; float p2_y = y + height;
	float p3_x = x + width; float p3_y = y + height;
	float p4_x = x + width; float p4_y = y - height;

	glBegin (GL_QUADS);
		glTexCoord2f (0.0 , 0.0); glVertex3f (p2_x, p2_y, 0.0);
		glTexCoord2f (tc_x, 0.0); glVertex3f (p3_x, p3_y, 0.0);
		glTexCoord2f (tc_x, tc_y); glVertex3f (p4_x, p4_y, 0.0);
		glTexCoord2f (0.0 , tc_y); glVertex3f (p1_x, p1_y, 0.0);
	glEnd ();

	/*
	glDisable (GL_TEXTURE_GEN_S);
	glDisable (GL_TEXTURE_GEN_T);
	*/
	glDisable (GL_TEXTURE_2D);
}


void RenderUtil::renderTruetypeString (float x, 
													float y, 
													int size,
													TextAlignment text_alignment,
													bool cache_text,
													const char* text)
{
	GLuint texture;
	TextureInfo* texture_info;

	try {
		TextTexture& text_texture = searchTextCache (text);
		texture = text_texture.texture;
		texture_info = &text_texture.texture_info;
	}
	catch (exception& e) {
		TextTexture text_texture;

		ResourceHandler* resource_handler = ResourceHandler::instance ();
   	text_texture.texture = resource_handler->recvFontString (text,
                 													"data/fonts/MuktiNarrow.ttf",
                 													false,
                 													size,
                 													&text_texture.texture_info);

		if (cache_text)
			cacheText (text, text_texture);

		texture = text_texture.texture;
		texture_info = &text_texture.texture_info;
	}

	GLfloat projection_matrix[4][4];
	glGetFloatv (GL_PROJECTION_MATRIX, &projection_matrix[0][0]);

	float p_width = 2 / projection_matrix[0][0];
	float p_height = -2 / projection_matrix[1][1];

	float width = texture_info->w  * p_width / 1600;
	float height = texture_info->h * p_height / 1200;

	float _x,_y;

	switch (text_alignment) {
		case left:
			_x = x + width;
			_y = y;
		break;

		case center:
			_x = x;
			_y = y;
		break;

		case right:
			_x = x - width;
			_y = y;
		break;
	}

	float p1_x = _x - width; float p1_y = _y - height;
	float p2_x = _x - width; float p2_y = _y + height;
	float p3_x = _x + width; float p3_y = _y + height;
	float p4_x = _x + width; float p4_y = _y - height;

	GLfloat texMinX = texture_info->texcoord[0];
	GLfloat texMinY = texture_info->texcoord[1];
	GLfloat texMaxX = texture_info->texcoord[2];
	GLfloat texMaxY = texture_info->texcoord[3];

	glEnable (GL_TEXTURE_2D);
	glColor4f (1.0, 1.0, 1.0, 1.0);
	glBindTexture (GL_TEXTURE_2D, texture);

	glBegin (GL_QUADS);
		glTexCoord2f (texMinX, texMaxY); glVertex2f (p2_x, p2_y);
		glTexCoord2f (texMaxX, texMaxY); glVertex2f (p3_x, p3_y);
		glTexCoord2f (texMaxX, texMinY); glVertex2f (p4_x, p4_y);
		glTexCoord2f (texMinX, texMinY); glVertex2f (p1_x, p1_y);
	glEnd ();

	glDisable (GL_TEXTURE_2D);

	if (!cache_text)
		glDeleteTextures (1, &texture);
}

void RenderUtil::renderBitmapString (float x,
											    float y,
 											    TextAlignment text_alignment,
												 const char* font_path,
												 const char* text)
{	
	BitmapFont* font;

	ResourceHandler* resource_handler = ResourceHandler::instance ();
   font = resource_handler->loadBitmapFont (font_path);

	glEnable (GL_TEXTURE_2D);

	float _x,_y;

	switch (text_alignment) {
		case left:
			_x = x - font->String_Width (string (text));
			_y = y - font->Line_Height () / 2;
		break;

		case center:
			_x = x - font->String_Width (string (text)) / 2;
			_y = y - font->Line_Height () / 2;
		break;

		case right:
			_x = x;
			_y = y - font->Line_Height () / 2;
		break;
	}

	font->Draw_String (string (text), _x, _y);

	glDisable (GL_TEXTURE_2D);
}

void RenderUtil::set2dMode (float width, float height)
{
	SDL_Surface *screen = SDL_GetVideoSurface();

   glPushAttrib(GL_ENABLE_BIT);
	glDisable(GL_DEPTH_TEST);
   glDisable(GL_CULL_FACE);
   glEnable(GL_TEXTURE_2D);

   glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

   glViewport(0, 0, screen->w, screen->h);

   glMatrixMode(GL_PROJECTION);
   glPushMatrix();
   glLoadIdentity();

	glOrtho(0.0, width, height, 0.0, 0.0, 1.0);

   glMatrixMode(GL_MODELVIEW);
   glPushMatrix();
   glLoadIdentity();

   glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
}

void RenderUtil::unset2dMode ()
{
	glMatrixMode(GL_MODELVIEW);
   glPopMatrix();

   glMatrixMode(GL_PROJECTION);
   glPopMatrix();

   glPopAttrib();

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable (GL_BLEND);
   
   glMatrixMode (GL_MODELVIEW);
}

void RenderUtil::cacheText (string text, TextTexture text_texture)
{
	if (text_cache.find (text) == text_cache.end ())
		text_cache[text] = text_texture;
}

RenderUtil::TextTexture& RenderUtil::searchTextCache (string text)
{
	map<string, TextTexture>::iterator it;

	if ((it = text_cache.find (text)) != text_cache.end ()) {
		return it->second;
	}
	else {
		throw domain_error ("RenderUtil::searchTextCache: cache miss.");
	}
}

void RenderUtil::clearCache ()
{
	map<string, TextTexture>::iterator it;
	for (it = text_cache.begin (); it != text_cache.end (); it++) {
				glDeleteTextures (1, &(it->second.texture));
	}

	text_cache.clear ();
}
