#ifndef RENDERUTIL_H
#define RENDERUTIL_H

#include <map>
#include <string>
#include <GL/gl.h>

#include "resourcehandler.h"

using namespace std;

class RenderUtil
{
	private:

	struct TextTexture {
		TextureInfo texture_info;
		GLuint texture;
	};

	public:

	enum TextAlignment {
		left,
		center,
		right
	};

	static void renderSquare (float x,
									  float y, 
									  float side,
									  const char* texture);

	static void renderSquareTrans (float x,
											 float y, 
											 float side,
											 float alpha,
											 const char* texture);

	static void renderCircle (float x,
									  float y,
									  float radius,
									  int detail, const char* texture);

	static void renderRectangle (float x,
										  float y,
										  float width,
										  float height,
										  const char* texture);

	static void renderRectangleTiled (float x,
												 float y,
												 float width,
												 float height,
												 float tiles,
												 const char* texture);

	static void renderRectangle (float x,
										  float y,
										  float width,
										  float height,
										  GLuint texture);

	static void renderTruetypeString (float x,
												 float y, 
												 int size,
												 TextAlignment pos,
												 bool cache_text,
												 const char* text);

	static void renderBitmapString (float x,
											  float y,
											  TextAlignment pos,
											  const char* font,
											  const char* text);

	static void set2dMode (float width, float height);
	static void unset2dMode ();
	static void clearCache ();

	private:

	static void cacheText (string text, TextTexture);
	static TextTexture& searchTextCache (string text);

	static map<string, TextTexture> text_cache;
};

#endif // RENDERUTIL_H
