#include <stdexcept>
#include <iostream>
#include <stdio.h>
#include <unistd.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

#include "resourcehandler.h"

/*
#define DATA_PATH "/usr/share/games/superbunguru/"
*/

ResourceHandler::ResourceHandler()
{
   if (TTF_Init () < 0)
      throw domain_error ("ResourceHandler: Couldn't initialize font library");

	// cwd datadir has precedence over installed datadir
	FILE* fp = fopen ("./superbunguru", "r");
	if (fp) {
		fclose (fp);
		path_prefix = "./";
		cout << "using local datadir" << endl;
	}
	else {
		path_prefix = DATA_PATH;
		cout << "using installed datadir" << endl;
	}

	cout << "==> ResourceHandler loaded <==" << endl;
}


ResourceHandler::~ResourceHandler()
{
	map<string,GLuint>::iterator tex_it;
   for (tex_it = textures.begin (); tex_it != textures.end (); tex_it++) {
		cout << "deleting: " << tex_it->first << endl;
		glDeleteTextures (1, &(tex_it->second));
	}
 
   map<string, TTF_Font*>::iterator fon_it;
   for (fon_it = fonts.begin (); fon_it != fonts.end (); fon_it++) {
      cout << "deleting: " << fon_it->first << endl;
      TTF_CloseFont (fon_it->second);
   }

	
	map<string, BitmapFont*>::iterator bfon_it;
   for (bfon_it = bitmap_fonts.begin (); bfon_it != bitmap_fonts.end (); bfon_it++) {
      cout << "deleting: " << bfon_it->first << endl;
      delete bfon_it->second;
   }
	
	
	textures.clear ();
   fonts.clear ();
	bitmap_fonts.clear ();
   
   TTF_Quit ();
}

ResourceHandler* ResourceHandler::instance ()
{
	static ResourceHandler inst;
	return &inst;
}

void ResourceHandler::loadTexture (const char* texture_name)
{
	SDL_Surface* image = IMG_Load ((path_prefix + texture_name).c_str ());
	GLuint texture_ref;
	
	if (image == NULL)
		throw domain_error ("ResourceHandler: " + string (texture_name) + " not found!");
	
	/*SDL_Surface* conv = SDL_CreateRGBSurface( SDL_SWSURFACE, image->w, image->h, 32,
                                 #if SDL_BYTEORDER == SDL_BIG_ENDIAN
                                         0xff000000, 0x00ff0000, 0x0000ff00, 0x000000ff);
                                 #else
                                     0x000000ff, 0x0000ff00, 0x00ff0000, 0xff000000);
                                 #endif  
	*/
		
	glGenTextures (1, &texture_ref);
	glBindTexture (GL_TEXTURE_2D, texture_ref);
	
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	
	glPixelStorei(GL_UNPACK_ROW_LENGTH, image->pitch / image->format->BytesPerPixel);
	
	glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, image->w, image->h, 
		      0, GL_RGBA, GL_UNSIGNED_BYTE, image->pixels);
	glPixelStorei( GL_UNPACK_ROW_LENGTH, 0 );
	
	textures.insert (pair<string,GLuint>(string (texture_name), texture_ref));
	
	SDL_FreeSurface (image);
	//SDL_FreeSurface (conv);
}


BitmapFont* ResourceHandler::loadBitmapFont (const char* font_path)
{
	// don't load the same font twice
	map<string, BitmapFont*>::iterator it = bitmap_fonts.find (string (font_path));
	if (it != bitmap_fonts.end ())
		return it->second;

	BitmapFont* font = new BitmapFont ((path_prefix + font_path).c_str ());

//	if (font == NULL) {
//      throw domain_error ("ResourceHandler: Couldn't load font, " + string (font_path));
//	}
   
   bitmap_fonts.insert (pair<string, BitmapFont*>(string (font_path), font));
	return font;
}

void ResourceHandler::loadFont (const char* font_name, int point_size)
{
	// don't load the same font twice
	map<string,TTF_Font*>::iterator it = fonts.find (string (font_name) + string ((char*)&point_size));
   if (it != fonts.end ())
       return;

   TTF_Font* font;
   font = TTF_OpenFont ((path_prefix + font_name).c_str (), point_size);

   if (font == NULL)
   {
      throw domain_error ("ResourceHandler: Couldn't load font, " + string (font_name));
   }
   
   fonts.insert (pair<string, TTF_Font*>(string (font_name) + string((char*)&point_size), font));
}

GLuint ResourceHandler::recvTexture (const char* texture_name)
{
	map<string,GLuint>::iterator it = textures.find (string (texture_name));
	
	if (it != textures.end ())
		return it->second;
	else {
		loadTexture (texture_name);
		return recvTexture (texture_name);
	}
}

/*
GLuint ResourceHandler::recvBitmapFontString (string text, const char* font_path, TextureInfo* texture_info)
{
	SFont_Font* font;
   map<string,SFont_Font*>::iterator it = bitmap_fonts.find (string (font_path));

   if (it != bitmap_fonts.end ())
       font = it->second;
   else {
      loadBitmapFont (font_path);
      return recvBitmapFontString (text, font_path, texture_info);
   }

	GLuint texture;

   SDL_Surface *rendered_text = createSurface (SFont_TextWidth (font, text.c_str ()), font->Surface->h);
	SFont_WriteCenter(rendered_text, font, 20, text.c_str ());
	
	texture_info->w = rendered_text->w;
	texture_info->h = rendered_text->h;

	texture = surfaceToTexture (rendered_text, texture_info);
	SDL_FreeSurface (rendered_text);

	return texture;
}
*/

GLuint ResourceHandler::recvFontString (string text, 
                                       const char* font_name,
                                       bool fast,
                                       int ptsize,
                                       TextureInfo* textinfo)
{
   TTF_Font* font;
   map<string,TTF_Font*>::iterator it = fonts.find (string (font_name) + string ((char*)&ptsize));

   if (it != fonts.end ())
       font = it->second;
   else {
      loadFont (font_name, ptsize);
      return recvFontString (text, font_name, fast, ptsize, textinfo);
   }

   GLuint texture;
   SDL_Color white = { 0xFF, 0xFF, 0xFF, 0 };

   TTF_SetFontStyle (font, TTF_STYLE_NORMAL);
   SDL_Surface *rendered_text;

   if (fast)
      rendered_text = TTF_RenderText_Solid (font, text.c_str (), white);
   else
      rendered_text = TTF_RenderText_Blended (font, text.c_str (), white);

   if (rendered_text == NULL)
      throw domain_error ("ResourceHandler: couldn't render text: " + text);

   textinfo->w = rendered_text->w;
   textinfo->h = rendered_text->h;
   texture = surfaceToTexture (rendered_text, textinfo);
   SDL_FreeSurface (rendered_text);

   return texture;
}

SDL_Surface* ResourceHandler::createSurface (int width, int height)
{
	/* Create a 32-bit surface with the bytes of each pixel in R,G,B,A order,
		as expected by OpenGL for textures */
	SDL_Surface *surface;
	Uint32 rmask, gmask, bmask, amask;

	/* SDL interprets each pixel as a 32-bit number, so our masks must depend
		on the endianness (byte order) of the machine */
	#if SDL_BYTEORDER == SDL_BIG_ENDIAN
   	 rmask = 0xff000000;
	    gmask = 0x00ff0000;
   	 bmask = 0x0000ff00;
	    amask = 0x000000ff;
	#else
   	 rmask = 0x000000ff;
	    gmask = 0x0000ff00;
   	 bmask = 0x00ff0000;
	    amask = 0xff000000;
	#endif

	surface = SDL_CreateRGBSurface(SDL_SWSURFACE, width, height, 32,
                                   rmask, gmask, bmask, amask);
	if (surface == NULL) {
		throw domain_error ("ResourceHandler: failed to create surface, " + string (SDL_GetError()));
	}

	cout << "Created Surface! (" << width << "," << height << ")" << endl;

	return surface;
}

GLuint ResourceHandler::surfaceToTexture (SDL_Surface* surface, TextureInfo* textinfo)
{
   GLuint texture;
   int w, h;
   SDL_Surface *image;
   SDL_Rect area;
   Uint32 saved_flags;
   Uint8  saved_alpha;

   /* Use the surface width and height expanded to powers of 2 */
   w = powerOfTwo(surface->w);
   h = powerOfTwo(surface->h);
   textinfo->texcoord[0] = 0.0f;        /* Min X */
   textinfo->texcoord[1] = 0.0f;        /* Min Y */
   textinfo->texcoord[2] = (GLfloat)surface->w / w; /* Max X */
   textinfo->texcoord[3] = (GLfloat)surface->h / h; /* Max Y */

   image = SDL_CreateRGBSurface(
         SDL_SWSURFACE,
         w, h,
         32,
#if SDL_BYTEORDER == SDL_LIL_ENDIAN /* OpenGL RGBA masks */
         0x000000FF,
         0x0000FF00,
         0x00FF0000,
         0xFF000000
#else
         0xFF000000,
         0x00FF0000,
         0x0000FF00,
         0x000000FF
#endif
                    );

   if ( image == NULL ) {
      throw domain_error ("ResourceHandler: failed to create surface, out of memory?");
   }

   /* Save the alpha blending attributes */
   saved_flags = surface->flags&(SDL_SRCALPHA|SDL_RLEACCELOK);
   saved_alpha = surface->format->alpha;
   if ( (saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA ) {
      SDL_SetAlpha(surface, 0, 0);
   }

   /* Copy the surface into the GL texture image */
   area.x = 0;
   area.y = 0;
   area.w = surface->w;
   area.h = surface->h;
   SDL_BlitSurface(surface, &area, image, &area);

   /* Restore the alpha blending attributes */
   if ( (saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA ) {
      SDL_SetAlpha(surface, saved_flags, saved_alpha);
   }

   /* Create an OpenGL texture for the image */
   glGenTextures(1, &texture);
   glBindTexture(GL_TEXTURE_2D, texture);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
   glTexImage2D(GL_TEXTURE_2D,
                0,
                GL_RGBA,
                w, h,
                0,
                GL_RGBA,
                GL_UNSIGNED_BYTE,
                image->pixels);
   SDL_FreeSurface(image); /* No longer needed */

   return texture;
}

/* Quick utility function for texture creation */
int ResourceHandler::powerOfTwo(int input)
{
   int value = 1;

   while ( value < input ) {
      value <<= 1;
   }
   return value;
}

