#ifndef RESOURCEHANDLER_H
#define RESOURCEHANDLER_H

#include <map>
#include <string>
#include <GL/gl.h>
#include <SDL/SDL_ttf.h>

#include "bitmapfont.h"


using namespace std;

struct TextureInfo
{
   int w, h;
   GLfloat texcoord[4];
};

/**
 * ResourceHandler
 * --------------
 * Handles loading of textures and fonts so far
 */
class ResourceHandler {
public:
	/**************
	 * Operations *
	 **************/
	
   /**
	 * returns the instance of the texture handler
	 */
   static ResourceHandler* instance ();

	/**
	 * returns the choosen data path
	 */
	string getDataPath () { return path_prefix; }
		
	/**
	 * open and load a texture from the given resource. if the resource
	 * isn't found an exception is thrown.
	 */
   void loadTexture (const char* texture_name);
 
   /**
    * open and load a font from the given resource. if the resource
    * isn't found an exception is thrown.
    */
   void loadFont (const char* font, int point_size);

   /**
    * open and load a font from the given resource. if the resource
    * isn't found an exception is thrown.
    */
	BitmapFont* loadBitmapFont (const char* font);

   /**
    * Returns a bitmap font instance.    
	GLFT_Font* recvBitmapFont (const char* font_name, int size);
	*/

	/**
	 * returns a previously loaded texture and if not found
	 * and exception is thrown.
	 */
   GLuint recvTexture (const char* texture_name);

   /**
    * takes a string and returns a texture with the
    * string rendered with the selected font. You'll
    * have to delete the texture your self.
    */
   GLuint recvFontString (string text, const char* font_name, bool fast, int ptsize, TextureInfo* textinfo);
	
protected:
	/****************************
	 * Constructors/Destructors *
	 ****************************/
	
   ResourceHandler ();
	~ResourceHandler();
    
private:

	SDL_Surface* createSurface (int w, int h);
   GLuint surfaceToTexture (SDL_Surface* surface, TextureInfo* texture_info);
   int powerOfTwo(int input);
   
	map<string, GLuint> textures;
   map<string, TTF_Font*> fonts;
	map<string, BitmapFont*> bitmap_fonts;

	string path_prefix;
};

#endif //RESOURCEHANDLER_H
