#include <iostream>
#include <cmath>
#include <string.h>

#include "rock.h"
#include "fallingrock.h"
//#include "particlesystem.h"
#include "renderutil.h"
#include "worldmodifier.h"

using namespace std;

Rock::Rock (float x, float y, Rocktype type)
	:
	BaseObject (&rockshape),
	startup (true),
	player_on_rock (false),
	rockshape (this)
{
	rockshape.x = x;
	rockshape.y = y;
	rockshape.sidelength = 0.2;

	this->type = type;

	switch (type) {
		case easy:
			damage = 1.0;
			break;
		case normal:
			damage = 0.5;
			break;
		case hard:
			damage = 0.3;
			break;
		case indestructible:
			damage = 999999.0;
			break;
	}

	boulders = new ParticleSystem ("data/gfx/ring_particle.png",
											 rockshape.x, rockshape.y, 0.05, 900, 0.1, 0.0, -M_PI/2.0, 1.0, 0.0, 2.0, 1.0, 0.4);
	boulders->active (false);

	smokepuffs = new ParticleSystem ("data/gfx/smoke.png",
												rockshape.x, rockshape.y + 0.2, 0.1, 900, 0.1, M_PI/10.0, M_PI/2.0, 0.2, 0.1, 0.4, 0.0, 0.3);
	smokepuffs->active (false);
}

Rock::Rock (TiXmlElement* rock_node)
	:
	BaseObject (&rockshape),
	startup (true),
	player_on_rock (false),
	rockshape (this)
{

	TiXmlElement* position_node = rock_node->FirstChildElement ("position");
	if (position_node) {
		position_node->QueryFloatAttribute ("x", &rockshape.x);
		position_node->QueryFloatAttribute ("y", &rockshape.y);
		position_node->QueryFloatAttribute ("sidelength", &rockshape.sidelength);
	}

	if (strcmp (rock_node->Attribute("type"), "easy") == 0) {
		damage = 1.0; type = easy; }
	if (strcmp (rock_node->Attribute("type"), "normal") == 0) {
		damage = 0.5; type = normal; }
	if (strcmp (rock_node->Attribute("type"), "hard") == 0) {
		damage = 0.3; type = hard; }
	if (strcmp (rock_node->Attribute("type"), "indestructible") == 0) {
		damage = 999999.0; type = indestructible; }

	boulders = new ParticleSystem ("data/gfx/ring_particle.png",
											 rockshape.x, rockshape.y, 0.02, 900, 0.1, 0.0, -M_PI/2.0, 1.0, 0.0, 2.0, 1.0, 0.4);
	boulders->active (false);

	smokepuffs = new ParticleSystem ("data/gfx/smoke.png",
												rockshape.x, rockshape.y + 0.2, 0.1, 900, 0.1, M_PI/4.0, M_PI/2.0, 0.05, 0.0, 0.8, 0.4, 0.3, 0.0, 0.2);
	smokepuffs->active (false);
}

Rock::~Rock ()
{
	/* empty */
}

void Rock::draw ()
{
	switch (type) {
		case indestructible:
			RenderUtil::renderSquare (rockshape.x,
								  			  rockshape.y,
								  			  rockshape.sidelength, "data/gfx/indestructible_rock.png");
		break;
		case easy:
			RenderUtil::renderSquare (rockshape.x,
											  rockshape.y,
											  rockshape.sidelength, "data/gfx/easyrock.png");
		break;
		case normal:
			RenderUtil::renderSquare (rockshape.x,
											  rockshape.y,
											  rockshape.sidelength, "data/gfx/normalrock.png");
		break;
		case hard:
			RenderUtil::renderSquare (rockshape.x,
											  rockshape.y,
											  rockshape.sidelength, "data/gfx/hardrock.png");
		break;
	}
}

TiXmlNode& Rock::getXMLRepresentation ()
{
	TiXmlElement* rock_node = new TiXmlElement ("rock");
	TiXmlElement position_node = ("position");

	switch (type) {
		case easy:
			rock_node->SetAttribute ("type", "easy");
			break;
		case normal:
			rock_node->SetAttribute ("type", "normal");
			break;
		case hard:
			rock_node->SetAttribute ("type", "hard");
			break;
		case indestructible:
			rock_node->SetAttribute ("type", "indestructible");
			break;
	}
	
	position_node.SetDoubleAttribute ("x", rockshape.x);
	position_node.SetDoubleAttribute ("y", rockshape.y);
	position_node.SetDoubleAttribute ("sidelength", rockshape.sidelength);

	rock_node->InsertEndChild (position_node);

	return *rock_node;
}

void Rock::update (float delta_t, WorldModifier& worldmodifier)
{
	if (startup && type != indestructible) {
		worldmodifier.insertFreeObject (boulders);
		worldmodifier.insertFreeObject (smokepuffs);
		startup = false;
		//boulders->active (false);
	}

	if (type != indestructible) {
		if (player_on_rock)
			player_on_rock = false;
		else {
			boulders->active (false);
			smokepuffs->active (false);
		}
	}
}

void Rock::RockShape::manipulate (Player* player, WorldModifier& worldmodifier, float delta_t)
{
	PlayerShape& playershape = player->getPlayerShape ();

	float delta_x = x - playershape.x;
	float delta_y = y - playershape.y;

	float max_delta = sidelength + playershape.sidelength;

	if (fabs(delta_x) > fabs(delta_y))	{
		if (delta_x > 0)
			playershape.x = x - sidelength - playershape.sidelength;
		else
			playershape.x = x + sidelength + playershape.sidelength;
	}
	else if (fabs(delta_y) > fabs(delta_x)) {
		if (fabs (delta_x) < max_delta*0.90) {				// stops you from standing on the edge
			if (delta_y > 0) {									// player hitting the rock from underneath
				if ((playershape.vertical_velocity > 9.0) && (rock_p->type != indestructible)) {
					worldmodifier.deleteObject (rock_p->getSignature ());
					worldmodifier.insertFreeObject (new ParticleSystem ("data/gfx/yellow_particle.png",
																						 x, y, 0.05, 30, 0.005, M_PI/4.0, M_PI/2.0, 0.0, 1.0, 2.0, 1.0, 0.4, 0.0, -10.0));
				}

				playershape.y = y - sidelength - playershape.sidelength;
			} 
			else {													// player standing on the rock
				playershape.y = y + sidelength + playershape.sidelength;
				player->rechargePower (delta_t);				

				if (rock_p->type != indestructible) {
					rock_p->damage -= delta_t;
					rock_p->smokepuffs->active (true);
					rock_p->boulders->active (true);
					rock_p->player_on_rock = true;
				}
			}
			
			playershape.vertical_velocity = 0.0;
		}
	}

	if (rock_p->damage <= 0.0) {
		rock_p->smokepuffs->shutdown ();
		rock_p->boulders->shutdown ();
		worldmodifier.deleteObject (rock_p->getSignature ());
		worldmodifier.insertFreeObject (new FallingRock (x, y, rock_p->type));
	}
}
