#include <cmath>
#include <iostream>

#include "textureanimation.h"
#include "resourcehandler.h"

TextureAnimation::TextureAnimation (string texture_path, int frames, int sets, float rate)
	:
	timer (0.0)
{
	ResourceHandler* resource_handler = ResourceHandler::instance ();
	texture = resource_handler->recvTexture (texture_path.c_str());

	this->frames = frames;
	this->sets = sets;
	this->rate = rate;

	selectSet (0);
}

TextureAnimation::~TextureAnimation ()
{
	clearAnimations ();
}

void TextureAnimation::selectSet (float set)
{
	setcoord1 = (set + 0.0) / sets;
	setcoord2 = (set + 1.0) / sets;
}

void TextureAnimation::selectFrame (int frame)
{
	this->frame = frame;
}

void TextureAnimation::draw (Position& pos, Outline& out)
{
		glEnable (GL_TEXTURE_2D);
		glColor4f (1.0, 1.0, 1.0, 1.0);
		glBindTexture (GL_TEXTURE_2D, texture);

		if (rate > 0.0) {
			timer += getIdleTime ();
			frame = (int)fabs(timer/rate)%frames;
		}

		float framecoord1 = (frame +0.0) / frames;
		float framecoord2 = (frame +1.0) / frames;

		glBegin (GL_QUADS);
			glTexCoord2f (framecoord1, setcoord1); glVertex3f (out.p2.x, out.p2.y, 0.0);
			glTexCoord2f (framecoord2, setcoord1); glVertex3f (out.p3.x, out.p3.y, 0.0);
			glTexCoord2f (framecoord2, setcoord2); glVertex3f (out.p4.x, out.p4.y, 0.0);
			glTexCoord2f (framecoord1, setcoord2); glVertex3f (out.p1.x, out.p1.y, 0.0);
		glEnd ();

		glDisable (GL_TEXTURE_2D);
}
