#include <iostream>

#include "worldmodifier.h"
#include "world.h"

#define MAX_SIGNATURES 300000000

WorldModifier::WorldModifier (World* _world)
	:
	world (_world)
{
	signature_count = 0;
}

WorldModifier::~WorldModifier ()
{
}

void WorldModifier::execute ()
{
	vector<long>::iterator it;
	for (it = objects_to_be_deleted.begin (); it != objects_to_be_deleted.end (); it++) {
		_deleteObject (*it);
	}
}

void WorldModifier::deleteObject (long signature)
{
	objects_to_be_deleted.push_back (signature);
}

void WorldModifier::_deleteObject (long signature)
{
	FreeObjects::iterator it;
	it = world->freeobjects.find (signature);

	if (it != world->freeobjects.end ()) {
		delete it->second;
		world->freeobjects.erase (it);
	} 
	else {
		deleteTileObject (signature);
	}
}

void WorldModifier::deleteTileObject (long signature)
{
	Tilemap::iterator row;
	FreeObjects::iterator object;
	for (row = world->tilemap.begin (); row != world->tilemap.end (); row++) {
		object = row->find (signature);

		if (object != row->end ()) {
			delete object->second;
			row->erase (object);
		}
	}
}

void WorldModifier::insertFreeObject (BaseObject* object)
{
	if (object->getSignature () != 0) {
		FreeObjects::iterator it;
		it = world->freeobjects.find (object->getSignature ());

		if (it != world->freeobjects.end ())
			return;
	}
	else {
		object->setSignature (generateSignature ());
	}

	world->freeobjects[object->getSignature ()] = object;
}


void WorldModifier::insertTileObject (BaseObject* object, FreeObjects& row)
{
	if (object->getSignature () != 0)
		return; // otherwise we would have to do a full scan
	else
		object->setSignature (generateSignature ());

	row[object->getSignature ()] = object;
}


long WorldModifier::generateSignature ()
{
	if (signature_count >= MAX_SIGNATURES)
		std::cout << "Warning! no safe unique signatures left" << endl;

	return ++signature_count;
}
